@@ -138,7 +138,7 @@ private:
     QVector<Frame> readAnimationFrames();
 
     Properties readProperties();
-    void readProperty(Properties *properties);
+    void readProperty(Properties *properties, const ExportContext &context);
 
     MapReader *p;
 
@@ -270,6 +270,15 @@ std::unique_ptr<Map> MapReaderPrivate::readMap()
     mapParameters.staggerAxis = staggerAxisFromString(staggerAxis);
     mapParameters.staggerIndex = staggerIndexFromString(staggerIndex);
 
+    bool ok;
+    const qreal parallaxOriginX = atts.value(QLatin1String("parallaxoriginx")).toDouble(&ok);
+    if (ok)
+        mapParameters.parallaxOrigin.setX(parallaxOriginX);
+
+    const qreal parallaxOriginY = atts.value(QLatin1String("parallaxoriginy")).toDouble(&ok);
+    if (ok)
+        mapParameters.parallaxOrigin.setY(parallaxOriginY);
+
     const QString backgroundColor = atts.value(QLatin1String("backgroundcolor")).toString();
     if (QColor::isValidColor(backgroundColor))
         mapParameters.backgroundColor = QColor(backgroundColor);
@@ -1142,6 +1151,9 @@ std::unique_ptr<ImageLayer> MapReaderPrivate::readImageLayer()
     auto imageLayer = std::make_unique<ImageLayer>(name, x, y);
     readLayerAttributes(*imageLayer, atts);
 
+    imageLayer->setRepeatX(atts.value(QLatin1String("repeatx")).toInt());
+    imageLayer->setRepeatY(atts.value(QLatin1String("repeaty")).toInt());
+
     // Image layer pixel position moved from x/y to offsetx/offsety for
     // consistency with other layers. This is here for backwards compatibility.
     if (!atts.hasAttribute(QLatin1String("offsetx")))
@@ -1413,10 +1425,11 @@ Properties MapReaderPrivate::readProperties()
     Q_ASSERT(xml.isStartElement() && xml.name() == QLatin1String("properties"));
 
     Properties properties;
+    const ExportContext context(mPath.path());
 
     while (xml.readNextStartElement()) {
         if (xml.name() == QLatin1String("property"))
-            readProperty(&properties);
+            readProperty(&properties, context);
         else
             readUnknownElement();
     }
@@ -1424,34 +1437,35 @@ Properties MapReaderPrivate::readProperties()
     return properties;
 }
 
-void MapReaderPrivate::readProperty(Properties *properties)
+void MapReaderPrivate::readProperty(Properties *properties, const ExportContext &context)
 {
     Q_ASSERT(xml.isStartElement() && xml.name() == QLatin1String("property"));
 
     const QXmlStreamAttributes atts = xml.attributes();
     QString propertyName = atts.value(QLatin1String("name")).toString();
-    QString propertyValue = atts.value(QLatin1String("value")).toString();
-    QString propertyType = atts.value(QLatin1String("type")).toString();
+
+    ExportValue exportValue;
+    exportValue.typeName = atts.value(QLatin1String("type")).toString();
+    exportValue.propertyTypeName = atts.value(QLatin1String("propertytype")).toString();
+
+    const QString propertyValue = atts.value(QLatin1String("value")).toString();
+    exportValue.value = propertyValue;
 
     while (xml.readNext() != QXmlStreamReader::Invalid) {
         if (xml.isEndElement()) {
             break;
         } else if (xml.isCharacters() && !xml.isWhitespace()) {
             if (propertyValue.isEmpty())
-                propertyValue = xml.text().toString();
+                exportValue.value = xml.text().toString();
         } else if (xml.isStartElement()) {
-            readUnknownElement();
+            if (xml.name() == QLatin1String("properties"))
+                exportValue.value = readProperties();
+            else
+                readUnknownElement();
         }
     }
 
-    QVariant variant(propertyValue);
-
-    if (!propertyType.isEmpty()) {
-        int type = nameToType(propertyType);
-        variant = fromExportValue(variant, type, mPath);
-    }
-
-    properties->insert(propertyName, variant);
+    properties->insert(propertyName, context.toPropertyValue(exportValue));
 }
 
 
