@@ -82,7 +82,18 @@ MapRenderer::~MapRenderer()
 
 QRectF MapRenderer::boundingRect(const ImageLayer *imageLayer) const
 {
-    return QRectF(QPointF(), imageLayer->image().size());
+    QRectF bounds = QRectF(QPointF(), imageLayer->image().size());
+
+    if (imageLayer->repeatX()) {
+        bounds.setLeft(INT_MIN / 512);
+        bounds.setRight(INT_MAX / 256);
+    }
+    if (imageLayer->repeatY()) {
+        bounds.setTop(INT_MIN / 512);
+        bounds.setBottom(INT_MAX / 256);
+    }
+
+    return bounds;
 }
 
 QPainterPath MapRenderer::pointShape(const QPointF &position) const
@@ -113,9 +124,14 @@ void MapRenderer::drawImageLayer(QPainter *painter,
                                  const ImageLayer *imageLayer,
                                  const QRectF &exposed) const
 {
-    Q_UNUSED(exposed)
-
-    painter->drawPixmap(QPointF(), tinted(imageLayer->image(), imageLayer->effectiveTintColor()));
+    painter->save();
+    painter->setBrush(tinted(imageLayer->image(), imageLayer->effectiveTintColor()));
+    painter->setPen(Qt::NoPen);
+    if (exposed.isNull())
+        painter->drawRect(boundingRect(imageLayer));
+    else
+        painter->drawRect(boundingRect(imageLayer) & exposed);
+    painter->restore();
 }
 
 void MapRenderer::drawPointObject(QPainter *painter, const QColor &color) const
@@ -174,6 +190,18 @@ QPainterPath MapRenderer::pointInteractionShape(const MapObject *object) const
     return path;
 }
 
+QPointF MapRenderer::snapToGrid(const QPointF &pixelCoords, int subdivisions) const
+{
+    QPointF tileCoords = pixelToTileCoords(pixelCoords);
+    if (subdivisions > 1) {
+        tileCoords = (tileCoords * subdivisions).toPoint();
+        tileCoords /= subdivisions;
+    } else {
+        tileCoords = tileCoords.toPoint();
+    }
+    return tileToPixelCoords(tileCoords);
+}
+
 void MapRenderer::drawTileLayer(QPainter *painter, const TileLayer *layer, const QRectF &exposed) const
 {
     const QSize tileSize = map()->tileSize();
@@ -260,7 +288,8 @@ std::unique_ptr<MapRenderer> MapRenderer::create(const Map *map)
 }
 
 void MapRenderer::setupGridPens(const QPaintDevice *device, QColor color,
-                                QPen &gridPen, QPen &majorGridPen)
+                                QPen &gridPen, QPen &majorGridPen, int gridSize,
+                                QSize gridMajor) const
 {
     const qreal devicePixelRatio = device->devicePixelRatioF();
 
@@ -270,15 +299,18 @@ void MapRenderer::setupGridPens(const QPaintDevice *device, QColor color,
     const qreal dpiScale = device->logicalDpiX() / 96.0;
 #endif
 
+    const auto majorGridSize = qMax(gridMajor.width(), gridMajor.height()) * gridSize;
     const qreal dashLength = std::ceil(2.0 * dpiScale);
+    const qreal autoAlpha = qBound(0.0, (gridSize * mPainterScale - 3) / 17.0, 1.0);
+    const qreal majorAutoAlpha = qBound(0.0, (majorGridSize * mPainterScale - 3) / 17.0, 1.0);
 
-    color.setAlpha(96);
+    color.setAlpha(96 * autoAlpha);
 
     gridPen = QPen(color, 1.0 * devicePixelRatio);
     gridPen.setCosmetic(true);
     gridPen.setDashPattern({dashLength, dashLength});
 
-    color.setAlpha(192);
+    color.setAlpha(96 * autoAlpha + 96 * majorAutoAlpha);
 
     majorGridPen = gridPen;
     majorGridPen.setColor(color);
@@ -333,7 +365,7 @@ void CellRenderer::render(const Cell &cell, const QPointF &screenPos, const QSiz
 {
     const Tile *tile = cell.tile();
 
-    if (tile)
+    if (tile && mRenderer->testFlag(ShowTileAnimations))
         tile = tile->currentFrameTile();
 
     if (!tile || tile->image().isNull()) {
