@@ -28,7 +28,6 @@
 #include "objectgroup.h"
 #include "objecttemplate.h"
 #include "properties.h"
-#include "terrain.h"
 #include "tile.h"
 #include "tilelayer.h"
 #include "tileset.h"
@@ -46,7 +45,10 @@ QVariant MapToVariantConverter::toVariant(const Map &map, const QDir &mapDir)
     QVariantMap mapVariant;
 
     mapVariant[QStringLiteral("type")] = QLatin1String("map");
-    mapVariant[QStringLiteral("version")] = (mVersion == 2) ? 1.4 : 1.1;
+    if (mVersion == 2)
+        mapVariant[QStringLiteral("version")] = QStringLiteral("1.8");
+    else
+        mapVariant[QStringLiteral("version")] = 1.1;
     mapVariant[QStringLiteral("tiledversion")] = QCoreApplication::applicationVersion();
     mapVariant[QStringLiteral("orientation")] = orientationToString(map.orientation());
     mapVariant[QStringLiteral("renderorder")] = renderOrderToString(map.renderOrder());
@@ -92,6 +94,11 @@ QVariant MapToVariantConverter::toVariant(const Map &map, const QDir &mapDir)
         mapVariant[QStringLiteral("staggerindex")] = staggerIndexToString(map.staggerIndex());
     }
 
+    if (!map.parallaxOrigin().isNull()) {
+        mapVariant[QStringLiteral("parallaxoriginx")] = map.parallaxOrigin().x();
+        mapVariant[QStringLiteral("parallaxoriginy")] = map.parallaxOrigin().y();
+    }
+
     const QColor bgColor = map.backgroundColor();
     if (bgColor.isValid())
         mapVariant[QStringLiteral("backgroundcolor")] = colorToString(bgColor);
@@ -132,7 +139,7 @@ QVariant MapToVariantConverter::toVariant(const ObjectTemplate &objectTemplate,
     mGidMapper.clear();
     if (Tileset *tileset = objectTemplate.object()->cell().tileset()) {
         unsigned firstGid = 1;
-        mGidMapper.insert(firstGid, tileset->sharedPointer());
+        mGidMapper.insert(firstGid, tileset->sharedFromThis());
         objectTemplateVariant[QStringLiteral("tileset")] = toVariant(*tileset, firstGid);
     }
 
@@ -151,7 +158,7 @@ QVariant MapToVariantConverter::toVariant(const Tileset &tileset,
 
         const QString &fileName = tileset.fileName();
         if (!fileName.isEmpty()) {
-            QString source = mDir.relativeFilePath(fileName);
+            const QString source = filePathRelativeTo(mDir, fileName);
             tilesetVariant[QStringLiteral("source")] = source;
 
             // Tileset is external, so no need to write any of the stuff below
@@ -162,7 +169,10 @@ QVariant MapToVariantConverter::toVariant(const Tileset &tileset,
         tilesetVariant[QStringLiteral("type")] = QLatin1String("tileset");
 
         // Include version in external tilesets
-        tilesetVariant[QStringLiteral("version")] = (mVersion == 2) ? 1.4 : 1.1;
+        if (mVersion == 2)
+            tilesetVariant[QStringLiteral("version")] = QStringLiteral("1.8");
+        else
+            tilesetVariant[QStringLiteral("version")] = 1.1;
         tilesetVariant[QStringLiteral("tiledversion")] = QCoreApplication::applicationVersion();
     }
 
@@ -228,8 +238,18 @@ QVariant MapToVariantConverter::toVariant(const Tileset &tileset,
         tilesetVariant[QStringLiteral("imageheight")] = tileset.imageHeight();
     }
 
-    // Write the properties, terrain, external image, object group and
-    // animation for those tiles that have them.
+    const auto transformationFlags = tileset.transformationFlags();
+    if (transformationFlags) {
+        tilesetVariant[QStringLiteral("transformations")] = QVariantMap {
+            { QStringLiteral("hflip"), transformationFlags.testFlag(Tileset::AllowFlipHorizontally) },
+            { QStringLiteral("vflip"), transformationFlags.testFlag(Tileset::AllowFlipVertically) },
+            { QStringLiteral("rotate"), transformationFlags.testFlag(Tileset::AllowRotate) },
+            { QStringLiteral("preferuntransformed"), transformationFlags.testFlag(Tileset::PreferUntransformed) },
+        };
+    }
+
+    // Write the properties, external image, object group and animation for
+    // those tiles that have them.
 
     // Used for version 1
     QVariantMap tilePropertiesVariant;
@@ -239,7 +259,9 @@ QVariant MapToVariantConverter::toVariant(const Tileset &tileset,
     // Used for version 2
     QVariantList tilesVariant;
 
-    for (const Tile *tile  : tileset.tiles()) {
+    const bool includeAllTiles = mVersion != 1 && tileset.anyTileOutOfOrder();
+
+    for (const Tile *tile : tileset.tiles()) {
         const Properties properties = tile->properties();
         QVariantMap tileVariant;
 
@@ -254,12 +276,6 @@ QVariant MapToVariantConverter::toVariant(const Tileset &tileset,
 
         if (!tile->type().isEmpty())
             tileVariant[QStringLiteral("type")] = tile->type();
-        if (tile->terrain() != 0xFFFFFFFF) {
-            QVariantList terrainIds;
-            for (int j = 0; j < 4; ++j)
-                terrainIds << QVariant(tile->cornerTerrainId(j));
-            tileVariant[QStringLiteral("terrain")] = terrainIds;
-        }
         if (tile->probability() != 1.0)
             tileVariant[QStringLiteral("probability")] = tile->probability();
         if (!tile->imageSource().isEmpty()) {
@@ -285,7 +301,7 @@ QVariant MapToVariantConverter::toVariant(const Tileset &tileset,
             tileVariant[QStringLiteral("animation")] = frameVariants;
         }
 
-        if (!tileVariant.empty()) {
+        if (includeAllTiles || !tileVariant.empty()) {
             if (mVersion == 1) {
                 tilesVariantMap[QString::number(tile->id())] = tileVariant;
             } else {
@@ -305,21 +321,6 @@ QVariant MapToVariantConverter::toVariant(const Tileset &tileset,
     else if (!tilesVariant.empty())
         tilesetVariant[QStringLiteral("tiles")] = tilesVariant;
 
-    // Write terrains
-    if (tileset.terrainCount() > 0) {
-        QVariantList terrainsVariant;
-        for (int i = 0; i < tileset.terrainCount(); ++i) {
-            Terrain *terrain = tileset.terrain(i);
-            const Properties &properties = terrain->properties();
-            QVariantMap terrainVariant;
-            terrainVariant[QStringLiteral("name")] = terrain->name();
-            terrainVariant[QStringLiteral("tile")] = terrain->imageTileId();
-            addProperties(terrainVariant, properties);
-            terrainsVariant << terrainVariant;
-        }
-        tilesetVariant[QStringLiteral("terrains")] = terrainsVariant;
-    }
-
     // Write the Wang sets
     if (tileset.wangSetCount() > 0) {
         QVariantList wangSetVariants;
@@ -336,12 +337,13 @@ QVariant MapToVariantConverter::toVariant(const Tileset &tileset,
 QVariant MapToVariantConverter::toVariant(const Properties &properties) const
 {
     QVariantMap variantMap;
+    const ExportContext context(mDir.path());
 
     Properties::const_iterator it = properties.constBegin();
     Properties::const_iterator it_end = properties.constEnd();
     for (; it != it_end; ++it) {
-        const QVariant value = toExportValue(it.value(), mDir);
-        variantMap[it.key()] = value;
+        const auto exportValue = context.toExportValue(it.value());
+        variantMap[it.key()] = exportValue.value;
     }
 
     return variantMap;
@@ -350,11 +352,15 @@ QVariant MapToVariantConverter::toVariant(const Properties &properties) const
 QVariant MapToVariantConverter::propertyTypesToVariant(const Properties &properties) const
 {
     QVariantMap variantMap;
+    const ExportContext context;  // path irrelevant since only typeName is used
 
     Properties::const_iterator it = properties.constBegin();
     Properties::const_iterator it_end = properties.constEnd();
-    for (; it != it_end; ++it)
-        variantMap[it.key()] = typeToName(it.value().userType());
+    for (; it != it_end; ++it) {
+        const auto exportValue = context.toExportValue(it.value());
+        // TODO: Support custom property types in json1? Maybe with a customPropertyTypesToVariant...
+        variantMap[it.key()] = exportValue.typeName;
+    }
 
     return variantMap;
 }
@@ -365,25 +371,13 @@ QVariant MapToVariantConverter::toVariant(const WangSet &wangSet) const
     QVariantMap wangSetVariant;
 
     wangSetVariant[QStringLiteral("name")] = wangSet.name();
+    wangSetVariant[QStringLiteral("type")] = wangSetTypeToString(wangSet.type());
     wangSetVariant[QStringLiteral("tile")] = wangSet.imageTileId();
 
-    QVariantList edgeColorVariants;
-    if (wangSet.edgeColorCount() > 1) {
-        for (int i = 1; i <= wangSet.edgeColorCount(); ++i) {
-            if (WangColor *wc = wangSet.edgeColorAt(i).data())
-                edgeColorVariants.append(toVariant(*wc));
-        }
-    }
-    wangSetVariant[QStringLiteral("edgecolors")] = edgeColorVariants;
-
-    QVariantList cornerColorVariants;
-    if (wangSet.cornerColorCount() > 1) {
-        for (int i = 1; i <= wangSet.cornerColorCount(); ++i) {
-            if (WangColor *wc = wangSet.cornerColorAt(i).data())
-                cornerColorVariants.append(toVariant(*wc));
-        }
-    }
-    wangSetVariant[QStringLiteral("cornercolors")] = cornerColorVariants;
+    QVariantList colorVariants;
+    for (int i = 1; i <= wangSet.colorCount(); ++i)
+        colorVariants.append(toVariant(*wangSet.colorAt(i)));
+    wangSetVariant[QStringLiteral("colors")] = colorVariants;
 
     QVariantList wangTileVariants;
     const auto wangTiles = wangSet.sortedWangTiles();
@@ -391,14 +385,11 @@ QVariant MapToVariantConverter::toVariant(const WangSet &wangSet) const
         QVariantMap wangTileVariant;
 
         QVariantList wangIdVariant;
-        for (int i = 0; i < 8; ++i)
+        for (int i = 0; i < WangId::NumIndexes; ++i)
             wangIdVariant.append(QVariant(wangTile.wangId().indexColor(i)));
 
         wangTileVariant[QStringLiteral("wangid")] = wangIdVariant;
-        wangTileVariant[QStringLiteral("tileid")] = wangTile.tile()->id();
-        wangTileVariant[QStringLiteral("hflip")] = wangTile.flippedHorizontally();
-        wangTileVariant[QStringLiteral("vflip")] = wangTile.flippedVertically();
-        wangTileVariant[QStringLiteral("dflip")] = wangTile.flippedAntiDiagonally();
+        wangTileVariant[QStringLiteral("tileid")] = wangTile.tileId();
 
         wangTileVariants.append(wangTileVariant);
     }
@@ -416,6 +407,7 @@ QVariant MapToVariantConverter::toVariant(const WangColor &wangColor) const
     colorVariant[QStringLiteral("name")] = wangColor.name();
     colorVariant[QStringLiteral("probability")] = wangColor.probability();
     colorVariant[QStringLiteral("tile")] = wangColor.imageId();
+    addProperties(colorVariant, wangColor.properties());
     return colorVariant;
 }
 
@@ -534,7 +526,7 @@ QVariant MapToVariantConverter::toVariant(const MapObject &object) const
     addProperties(objectVariant, object.properties());
 
     if (const ObjectTemplate *objectTemplate = object.objectTemplate()) {
-        QString relativeFileName = mDir.relativeFilePath(objectTemplate->fileName());
+        const QString relativeFileName = filePathRelativeTo(mDir, objectTemplate->fileName());
         objectVariant[QStringLiteral("template")] = relativeFileName;
     }
 
@@ -679,6 +671,11 @@ QVariant MapToVariantConverter::toVariant(const ImageLayer &imageLayer) const
     if (transColor.isValid())
         imageLayerVariant[QStringLiteral("transparentcolor")] = transColor.name();
 
+    if (imageLayer.repeatX())
+        imageLayerVariant[QStringLiteral("repeatx")] = imageLayer.repeatX();
+    if (imageLayer.repeatY())
+        imageLayerVariant[QStringLiteral("repeaty")] = imageLayer.repeatY();
+
     return imageLayerVariant;
 }
 
@@ -746,6 +743,12 @@ void MapToVariantConverter::addLayerAttributes(QVariantMap &layerVariant,
         layerVariant[QStringLiteral("offsety")] = offset.y();
     }
 
+    const QPointF parallaxFactor = layer.parallaxFactor();
+    if (parallaxFactor.x() != 1.0)
+        layerVariant[QStringLiteral("parallaxx")] = parallaxFactor.x();
+    if (parallaxFactor.y() != 1.0)
+        layerVariant[QStringLiteral("parallaxy")] = parallaxFactor.y();
+
     if (layer.tintColor().isValid())
         layerVariant[QStringLiteral("tintcolor")] = colorToString(layer.tintColor());
 
@@ -758,6 +761,8 @@ void MapToVariantConverter::addProperties(QVariantMap &variantMap,
     if (properties.isEmpty())
         return;
 
+    const ExportContext context(mDir.path());
+
     if (mVersion == 1) {
         QVariantMap propertiesMap;
         QVariantMap propertyTypesMap;
@@ -765,11 +770,11 @@ void MapToVariantConverter::addProperties(QVariantMap &variantMap,
         Properties::const_iterator it = properties.constBegin();
         Properties::const_iterator it_end = properties.constEnd();
         for (; it != it_end; ++it) {
-            int type = it.value().userType();
-            const QVariant value = toExportValue(it.value(), mDir);
+            const auto exportValue = context.toExportValue(it.value());
 
-            propertiesMap[it.key()] = value;
-            propertyTypesMap[it.key()] = typeToName(type);
+            propertiesMap[it.key()] = exportValue.value;
+            propertyTypesMap[it.key()] = exportValue.typeName;
+            // TODO: Support custom property types in json1? Maybe with a customPropertyTypesMap...
         }
 
         variantMap[QStringLiteral("properties")] = propertiesMap;
@@ -780,13 +785,16 @@ void MapToVariantConverter::addProperties(QVariantMap &variantMap,
         Properties::const_iterator it = properties.constBegin();
         Properties::const_iterator it_end = properties.constEnd();
         for (; it != it_end; ++it) {
-            int type = it.value().userType();
-            const QVariant value = toExportValue(it.value(), mDir);
+            const auto exportValue = context.toExportValue(it.value());
 
             QVariantMap propertyVariantMap;
             propertyVariantMap[QStringLiteral("name")] = it.key();
-            propertyVariantMap[QStringLiteral("value")] = value;
-            propertyVariantMap[QStringLiteral("type")] = typeToName(type);
+            propertyVariantMap[QStringLiteral("value")] = exportValue.value;
+            propertyVariantMap[QStringLiteral("type")] = exportValue.typeName;
+
+            if (!exportValue.propertyTypeName.isEmpty())
+                propertyVariantMap[QStringLiteral("propertytype")] = exportValue.propertyTypeName;
+
             propertiesVariantList << propertyVariantMap;
         }
 
