@@ -36,7 +36,7 @@
 namespace Tiled {
 
 ObjectTypes Object::mObjectTypes;
-
+SharedPropertyTypes Object::mPropertyTypes;
 Object::~Object()
 {}
 
@@ -145,4 +145,18 @@ void Object::setObjectTypes(const ObjectTypes &objectTypes)
     mObjectTypes = objectTypes;
 }
 
+void Object::setPropertyTypes(const SharedPropertyTypes &propertyTypes)
+{
+    mPropertyTypes = propertyTypes;
+}
+
+const PropertyTypes &Object::propertyTypes()
+{
+    if (mPropertyTypes)
+        return *mPropertyTypes;
+
+    static PropertyTypes noTypes;
+    return noTypes;
+}
+
 } // namespace Tiled
