@@ -81,6 +81,14 @@ std::unique_ptr<Map> VariantToMapConverter::toMap(const QVariant &variant,
     mapParameters.staggerAxis = staggerAxisFromString(staggerAxis);
     mapParameters.staggerIndex = staggerIndexFromString(staggerIndex);
 
+    bool ok;
+    const qreal parallaxOriginX = variantMap[QStringLiteral("parallaxoriginx")].toDouble(&ok);
+    if (ok)
+        mapParameters.parallaxOrigin.setX( parallaxOriginX);
+    const qreal parallaxOriginY = variantMap[QStringLiteral("parallaxoriginy")].toDouble(&ok);
+    if (ok)
+        mapParameters.parallaxOrigin.setY(parallaxOriginY);
+
     const QString bgColor = variantMap[QStringLiteral("backgroundcolor")].toString();
     if (QColor::isValidColor(bgColor))
         mapParameters.backgroundColor = QColor(bgColor);
@@ -121,7 +129,6 @@ std::unique_ptr<Map> VariantToMapConverter::toMap(const QVariant &variant,
             tileset->loadImage();
     }
 
-    bool ok;
     const int compressionLevel = variantMap[QStringLiteral("compressionlevel")].toInt(&ok);
     if (ok)
         map->setCompressionLevel(compressionLevel);
@@ -156,18 +163,20 @@ Properties VariantToMapConverter::toProperties(const QVariant &propertiesVariant
 {
     Properties properties;
 
+    const ExportContext context(mDir.path());
+
     // read object-based format (1.0)
     const QVariantMap propertiesMap = propertiesVariant.toMap();
     const QVariantMap propertyTypesMap = propertyTypesVariant.toMap();
     QVariantMap::const_iterator it = propertiesMap.constBegin();
     QVariantMap::const_iterator it_end = propertiesMap.constEnd();
     for (; it != it_end; ++it) {
-        int type = nameToType(propertyTypesMap.value(it.key()).toString());
-        if (type == QMetaType::UnknownType)
-            type = QMetaType::QString;
+        ExportValue exportValue;
+        exportValue.value = it.value();
+        exportValue.typeName = propertyTypesMap.value(it.key()).toString();
+        // TODO: Support for custom property types with customPropertyTypesMap
 
-        const QVariant value = fromExportValue(it.value(), type, mDir);
-        properties[it.key()] = value;
+        properties[it.key()] = context.toPropertyValue(exportValue);
     }
 
     // read array-based format (1.2)
@@ -175,12 +184,12 @@ Properties VariantToMapConverter::toProperties(const QVariant &propertiesVariant
     for (const QVariant &propertyVariant : propertiesList) {
         const QVariantMap propertyVariantMap = propertyVariant.toMap();
         const QString propertyName = propertyVariantMap[QStringLiteral("name")].toString();
-        const QString propertyType = propertyVariantMap[QStringLiteral("type")].toString();
-        const QVariant propertyValue = propertyVariantMap[QStringLiteral("value")];
-        int type = nameToType(propertyType);
-        if (type == QMetaType::UnknownType)
-            type = QMetaType::QString;
-        properties[propertyName] = fromExportValue(propertyValue, type, mDir);
+        ExportValue exportValue;
+        exportValue.value = propertyVariantMap[QStringLiteral("value")];
+        exportValue.typeName = propertyVariantMap[QStringLiteral("type")].toString();
+        exportValue.propertyTypeName = propertyVariantMap[QStringLiteral("propertytype")].toString();
+
+        properties[propertyName] = context.toPropertyValue(exportValue);
     }
 
     return properties;
@@ -593,7 +602,7 @@ std::unique_ptr<TileLayer> VariantToMapConverter::toTileLayer(const QVariantMap
     const int startY = variantMap[QStringLiteral("starty")].toInt();
     const QVariant dataVariant = variantMap[QStringLiteral("data")];
 
-    typedef std::unique_ptr<TileLayer> TileLayerPtr;
+    using TileLayerPtr = std::unique_ptr<TileLayer>;
     TileLayerPtr tileLayer(new TileLayer(name,
                                          variantMap[QStringLiteral("x")].toInt(),
                                          variantMap[QStringLiteral("y")].toInt(),
@@ -648,7 +657,7 @@ std::unique_ptr<TileLayer> VariantToMapConverter::toTileLayer(const QVariantMap
 
 std::unique_ptr<ObjectGroup> VariantToMapConverter::toObjectGroup(const QVariantMap &variantMap)
 {
-    typedef std::unique_ptr<ObjectGroup> ObjectGroupPtr;
+    using ObjectGroupPtr = std::unique_ptr<ObjectGroup>;
     ObjectGroupPtr objectGroup(new ObjectGroup(variantMap[QStringLiteral("name")].toString(),
                                                variantMap[QStringLiteral("x")].toInt(),
                                                variantMap[QStringLiteral("y")].toInt()));
@@ -735,12 +744,12 @@ std::unique_ptr<MapObject> VariantToMapConverter::toMapObject(const QVariantMap
     const QVariant pointVariant = variantMap[QStringLiteral("point")];
     const QVariant textVariant = variantMap[QStringLiteral("text")];
 
-    if (polygonVariant.userType() == QVariant::List) {
+    if (polygonVariant.userType() == QMetaType::QVariantList) {
         object->setShape(MapObject::Polygon);
         object->setPolygon(toPolygon(polygonVariant));
         object->setPropertyChanged(MapObject::ShapeProperty);
     }
-    if (polylineVariant.userType() == QVariant::List) {
+    if (polylineVariant.userType() == QMetaType::QVariantList) {
         object->setShape(MapObject::Polyline);
         object->setPolygon(toPolygon(polylineVariant));
         object->setPropertyChanged(MapObject::ShapeProperty);
@@ -753,7 +762,7 @@ std::unique_ptr<MapObject> VariantToMapConverter::toMapObject(const QVariantMap
         object->setShape(MapObject::Point);
         object->setPropertyChanged(MapObject::ShapeProperty);
     }
-    if (textVariant.userType() == QVariant::Map) {
+    if (textVariant.userType() == QMetaType::QVariantMap) {
         object->setTextData(toTextData(textVariant.toMap()));
         object->setShape(MapObject::Text);
         object->setPropertyChanged(MapObject::TextProperty);
@@ -766,7 +775,7 @@ std::unique_ptr<MapObject> VariantToMapConverter::toMapObject(const QVariantMap
 
 std::unique_ptr<ImageLayer> VariantToMapConverter::toImageLayer(const QVariantMap &variantMap)
 {
-    typedef std::unique_ptr<ImageLayer> ImageLayerPtr;
+    using ImageLayerPtr = std::unique_ptr<ImageLayer>;
     ImageLayerPtr imageLayer(new ImageLayer(variantMap[QStringLiteral("name")].toString(),
                                             variantMap[QStringLiteral("x")].toInt(),
                                             variantMap[QStringLiteral("y")].toInt()));
