@@ -256,7 +256,7 @@ cls_mapobject = tiled.add_class('MapObject', cls_object)
 cls_mapobject.add_constructor([])
 cls_mapobject.add_constructor([('QString','name'), ('QString','type'),
     ('QPointF','pos'), ('QSizeF','size') ])
-cls_mapobject.add_enum('Shape', ('Rectangle','Polygon','Polyline'))
+cls_mapobject.add_enum('Shape', ('Rectangle','Polygon','Polyline','Ellipse','Text','Point'))
 cls_mapobject.add_method('setPosition', None, [('QPointF','pos')])
 cls_mapobject.add_method('x', 'double', [])
 cls_mapobject.add_method('setX', None, [('double','x')])
@@ -323,6 +323,8 @@ cls_layer.add_method('setX', None, [('int','x')])
 cls_layer.add_method('y', 'int', [])
 cls_layer.add_method('setY', None, [('int','y')])
 cls_layer.add_method('setPosition', None, [('int','x'),('int','y')])
+cls_layer.add_method('offset', 'QPointF', [])
+cls_layer.add_method('setOffset', None, [('QPointF','offset')])
 cls_layer.add_method('isTileLayer', 'bool', [])
 cls_layer.add_method('isObjectGroup', 'bool', [])
 cls_layer.add_method('isImageLayer', 'bool', [])
