@@ -19,8 +19,7 @@
  * this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef REPLICAISLAND_GLOBAL_H
-#define REPLICAISLAND_GLOBAL_H
+#pragma once
 
 #include <QtCore/qglobal.h>
 
@@ -29,5 +28,3 @@
 #else
 #  define REPLICAISLANDSHARED_EXPORT Q_DECL_IMPORT
 #endif
-
-#endif // REPLICAISLAND_GLOBAL_H
