@@ -49,11 +49,11 @@ AboutDialog::AboutDialog(QWidget *parent): QDialog(parent)
     const QString html = QCoreApplication::translate(
             "AboutDialog",
             "<p align=\"center\"><font size=\"+2\"><b>Tiled Map Editor</b></font><br><i>Version %1</i></p>\n"
-            "<p align=\"center\">Copyright 2008-2020 Thorbj&oslash;rn Lindeijer<br>(see the AUTHORS file for a full list of contributors)</p>\n"
+            "<p align=\"center\">Copyright 2008-%2 Thorbj&oslash;rn Lindeijer<br>(see the AUTHORS file for a full list of contributors)</p>\n"
             "<p align=\"center\">You may modify and redistribute this program under the terms of the GPL (version 2 or later). "
             "A copy of the GPL is contained in the 'COPYING' file distributed with Tiled.</p>\n"
             "<p align=\"center\"><a href=\"https://www.mapeditor.org/\">https://www.mapeditor.org/</a></p>\n")
-            .arg(QApplication::applicationVersion());
+            .arg(QApplication::applicationVersion(), QStringLiteral("2022"));
 
     textBrowser->setHtml(html);
 
@@ -72,3 +72,5 @@ void AboutDialog::donate()
 {
     QDesktopServices::openUrl(QUrl(QLatin1String("https://www.mapeditor.org/donate")));
 }
+
+#include "moc_aboutdialog.cpp"
