@@ -59,11 +59,13 @@ AbstractTileTool::~AbstractTileTool()
 void AbstractTileTool::activate(MapScene *scene)
 {
     scene->addItem(mBrushItem);
+    AbstractTool::activate(scene);
 }
 
 void AbstractTileTool::deactivate(MapScene *scene)
 {
     scene->removeItem(mBrushItem);
+    AbstractTool::deactivate(scene);
 }
 
 void AbstractTileTool::mouseEntered()
@@ -81,8 +83,9 @@ void AbstractTileTool::mouseMoved(const QPointF &pos, Qt::KeyboardModifiers)
     // Take into account the offset of the current layer
     QPointF offsetPos = pos;
     if (Layer *layer = currentLayer()) {
-        offsetPos -= layer->totalOffset();
-        mBrushItem->setLayerOffset(layer->totalOffset());
+        QPointF layerOffset = mapScene()->absolutePositionForLayer(*layer);
+        offsetPos -= layerOffset;
+        mBrushItem->setLayerOffset(layerOffset);
     }
 
     const MapRenderer *renderer = mapDocument()->renderer();
@@ -106,7 +109,9 @@ void AbstractTileTool::mouseMoved(const QPointF &pos, Qt::KeyboardModifiers)
 void AbstractTileTool::mousePressed(QGraphicsSceneMouseEvent *event)
 {
     if (event->button() == Qt::RightButton && event->modifiers() & Qt::ControlModifier) {
-        const QPoint pos = tilePosition();
+        const QPointF mousePos = event->pos();
+        const MapRenderer *renderer = mapDocument()->renderer();
+
         QList<Layer*> layers;
 
         const bool append = event->modifiers() & Qt::ShiftModifier;
@@ -121,7 +126,12 @@ void AbstractTileTool::mousePressed(QGraphicsSceneMouseEvent *event)
             if (tileLayer->isHidden())
                 continue;
 
-            if (!tileLayer->cellAt(pos - tileLayer->position()).isEmpty()) {
+            const QPointF layerOffset = mapScene()->absolutePositionForLayer(*tileLayer);
+            const QPointF tilePosF = renderer->screenToTileCoords(mousePos - layerOffset);
+            const QPoint tilePos = QPoint(qFloor(tilePosF.x()),
+                                          qFloor(tilePosF.y()));
+
+            if (!tileLayer->cellAt(tilePos - tileLayer->position()).isEmpty()) {
                 if (!layers.contains(tileLayer))
                     layers.append(tileLayer);
                 else if (append)
@@ -207,7 +217,7 @@ void AbstractTileTool::updateBrushVisibility()
     if (mBrushVisible) {
         const auto layers = targetLayers();
         for (auto layer : layers) {
-            if (!layer->isHidden()) {
+            if (!layer || !layer->isHidden()) {
                 showBrush = true;
                 break;
             }
@@ -216,6 +226,13 @@ void AbstractTileTool::updateBrushVisibility()
     mBrushItem->setVisible(showBrush);
 }
 
+/**
+ * Returns the target layers. The preview is automatically hidden when none of
+ * the target layers are visible.
+ *
+ * The result may include a nullptr, which indicates a layer is about to be
+ * created (which is assumed to be visible).
+ */
 QList<Layer *> AbstractTileTool::targetLayers() const
 {
     // By default, only a current tile layer is considered the target
@@ -235,24 +252,18 @@ QList<Layer *> AbstractTileTool::targetLayersForStamp(const TileStamp &stamp) co
     if (!mapDocument())
         return layers;
 
-    const Map &map = *mapDocument()->map();
+    QList<const TileLayer *> sourceLayers;
 
     for (const TileStampVariation &variation : stamp.variations()) {
-        LayerIterator it(variation.map, Layer::TileLayerType);
-        const Layer *firstLayer = it.next();
-        const bool isMultiLayer = firstLayer && it.next();
+        for (const Layer *layer : variation.map->tileLayers())
+            sourceLayers.append(static_cast<const TileLayer*>(layer));
 
-        if (isMultiLayer && !firstLayer->name().isEmpty()) {
-            for (Layer *layer : variation.map->tileLayers()) {
-                TileLayer *target = static_cast<TileLayer*>(map.findLayer(layer->name(), Layer::TileLayerType));
-                if (!layers.contains(target))
-                    layers.append(target);
-            }
-        } else {
-            if (TileLayer *tileLayer = currentTileLayer())
-                if (!layers.contains(tileLayer))
-                    layers.append(tileLayer);
-        }
+        const auto targetLayers = mapDocument()->findTargetLayers(sourceLayers);
+        for (TileLayer *target : targetLayers)
+            if (!layers.contains(target))
+                layers.append(target);
+
+        sourceLayers.clear();
     }
 
     return layers;
@@ -267,3 +278,5 @@ void AbstractTileTool::setBrushVisible(bool visible)
     updateStatusInfo();
     updateBrushVisibility();
 }
+
+#include "moc_abstracttiletool.cpp"
