@@ -67,6 +67,9 @@ class AbstractTool : public QObject
     Q_PROPERTY(QString statusInfo READ statusInfo WRITE setStatusInfo NOTIFY statusInfoChanged)
     Q_PROPERTY(QCursor cursor READ cursor WRITE setCursor NOTIFY cursorChanged)
     Q_PROPERTY(bool enabled READ isEnabled WRITE setEnabled NOTIFY enabledChanged)
+    Q_PROPERTY(bool visible READ isVisible WRITE setVisible NOTIFY visibleChanged)
+    Q_PROPERTY(bool usesSelectedTiles READ usesSelectedTiles WRITE setUsesSelectedTiles)
+    Q_PROPERTY(bool usesWangSets READ usesWangSets WRITE setUsesWangSets)
 
 public:
     /**
@@ -102,6 +105,12 @@ public:
     bool isVisible() const;
     void setVisible(bool visible);
 
+    bool usesSelectedTiles() const;
+    void setUsesSelectedTiles(bool usesSelectedTiles);
+
+    bool usesWangSets() const;
+    void setUsesWangSets(bool usesWangSets);
+
     ToolManager *toolManager() const;
     Tile *tile() const;
     ObjectTemplate *objectTemplate() const;
@@ -157,8 +166,7 @@ public:
 
     /**
      * Called when the user presses or releases a modifier key resulting
-     * in a change of modifier status, and when the tool is enabled with
-     * a modifier key pressed.
+     * in a change of modifier status, and after the tool is activated.
      */
     virtual void modifiersChanged(Qt::KeyboardModifiers) {}
 
@@ -221,6 +229,8 @@ private:
     Id mId;
     bool mEnabled = false;
     bool mVisible = true;
+    bool mUsesSelectedTiles = false;
+    bool mUsesWangSets = false;
 
     ToolManager *mToolManager = nullptr;
     MapDocument *mMapDocument = nullptr;
@@ -274,6 +284,26 @@ inline bool AbstractTool::isVisible() const
     return mVisible;
 }
 
+inline bool AbstractTool::usesSelectedTiles() const
+{
+    return mUsesSelectedTiles;
+}
+
+inline void AbstractTool::setUsesSelectedTiles(bool usesSelectedTiles)
+{
+    mUsesSelectedTiles = usesSelectedTiles;
+}
+
+inline bool AbstractTool::usesWangSets() const
+{
+    return mUsesWangSets;
+}
+
+inline void AbstractTool::setUsesWangSets(bool usesWangSets)
+{
+    mUsesWangSets = usesWangSets;
+}
+
 /**
  * Returns the ToolManager with which this tool is registered, if any.
  */
@@ -285,4 +315,3 @@ inline ToolManager *AbstractTool::toolManager() const
 } // namespace Tiled
 
 Q_DECLARE_METATYPE(Tiled::AbstractTool*)
-Q_DECLARE_INTERFACE(Tiled::AbstractTool, "org.mapeditor.AbstractTool")
