@@ -28,8 +28,15 @@ namespace Tiled {
 
 ActionManager::ActionManager(QObject *parent)
     : QObject(parent)
+    , mMenuSeparatorsParent(new QObject)
 {
     readCustomShortcuts();
+
+    mIdToMenu.insert(MenuIds::layerViewLayers, nullptr);
+    mIdToMenu.insert(MenuIds::mapViewObjects, nullptr);
+    mIdToMenu.insert(MenuIds::projectViewFiles, nullptr);
+    mIdToMenu.insert(MenuIds::propertiesViewProperties, nullptr);
+    mIdToMenu.insert(MenuIds::tilesetViewTiles, nullptr);
 }
 
 ActionManager::~ActionManager() = default;
@@ -44,7 +51,7 @@ void ActionManager::registerAction(QAction *action, Id id)
 {
     auto d = instance();
 
-    Q_ASSERT_X(!d->mIdToActions.contains(id, action), "ActionManager::registerAction", "duplicate action");
+    Q_ASSERT_X(!d->mIdToActions.contains(id, action), "ActionManager::registerAction", "action already registered");
     d->mIdToActions.insert(id, action);
     d->mLastKnownShortcuts.insert(id, action->shortcut());
 
@@ -98,6 +105,9 @@ void ActionManager::registerMenu(QMenu *menu, Id id)
 
     Q_ASSERT_X(!d->mIdToMenu.contains(id), "ActionManager::registerMenu", "duplicate id");
     d->mIdToMenu.insert(id, menu);
+
+    if (menu)
+        applyMenuExtensions(menu, id);
 }
 
 void ActionManager::unregisterMenu(Id id)
@@ -108,6 +118,32 @@ void ActionManager::unregisterMenu(Id id)
     d->mIdToMenu.remove(id);
 }
 
+void ActionManager::registerMenuExtension(Id id, MenuExtension extension)
+{
+    auto d = instance();
+    d->mIdToMenuExtensions[id].append(extension);
+
+    if (QMenu *menu = instance()->mIdToMenu.value(id))
+        d->applyMenuExtension(menu, extension);
+}
+
+void ActionManager::applyMenuExtensions(QMenu *menu, Id id)
+{
+    auto d = instance();
+
+    Q_ASSERT_X(d->mIdToMenu.contains(id), "ActionManager::applyMenuExtensions", "unknown id");
+    const auto extensions = d->mIdToMenuExtensions.value(id);
+    for (const auto &extension : extensions)
+        d->applyMenuExtension(menu, extension);
+}
+
+void ActionManager::clearMenuExtensions()
+{
+    auto d = instance();
+    d->mIdToMenuExtensions.clear();
+    d->mMenuSeparatorsParent.reset(new QObject);
+}
+
 QAction *ActionManager::action(Id id)
 {
     auto d = instance();
@@ -124,18 +160,9 @@ QAction *ActionManager::findAction(Id id)
     return d->mIdToActions.value(id);
 }
 
-QMenu *ActionManager::menu(Id id)
+bool ActionManager::hasMenu(Id id)
 {
-    auto d = instance();
-
-    auto menu = d->mIdToMenu.value(id);
-    Q_ASSERT_X(menu, "ActionManager::menu", "unknown id");
-    return menu;
-}
-
-QMenu *ActionManager::findMenu(Id id)
-{
-    return instance()->mIdToMenu.value(id);
+    return instance()->mIdToMenu.contains(id);
 }
 
 QList<Id> ActionManager::actions()
@@ -285,4 +312,21 @@ void ActionManager::updateToolTipWithShortcut(QAction *action)
     mApplyingToolTipWithShortcut = false;
 }
 
+void ActionManager::applyMenuExtension(QMenu *menu, const ActionManager::MenuExtension &extension)
+{
+    QAction *before = nullptr;
+
+    for (const MenuItem &item : extension.items) {
+        if (item.beforeAction)
+            before = ActionManager::findAction(item.beforeAction);
+
+        if (item.isSeparator)
+            menu->insertSeparator(before)->setParent(mMenuSeparatorsParent.get());
+        else
+            menu->insertAction(before, ActionManager::action(item.action));
+    }
+}
+
 } // namespace Tiled
+
+#include "moc_actionmanager.cpp"
