@@ -42,8 +42,6 @@ SessionOption<bool> AutomappingManager::automappingWhileDrawing { "automapping.w
 
 AutomappingManager::AutomappingManager(QObject *parent)
     : QObject(parent)
-    , mMapDocument(nullptr)
-    , mLoaded(false)
 {
     connect(&mWatcher, &QFileSystemWatcher::fileChanged,
             this, &AutomappingManager::onFileChanged);
@@ -87,7 +85,7 @@ void AutomappingManager::autoMapRegion(const QRegion &region)
     autoMapInternal(region, nullptr);
 }
 
-void AutomappingManager::onRegionEdited(const QRegion &where, Layer *touchedLayer)
+void AutomappingManager::onRegionEdited(const QRegion &where, TileLayer *touchedLayer)
 {
     if (automappingWhileDrawing)
         autoMapInternal(where, touchedLayer);
@@ -100,16 +98,23 @@ void AutomappingManager::onMapFileNameChanged()
 }
 
 void AutomappingManager::autoMapInternal(const QRegion &where,
-                                         Layer *touchedLayer)
+                                         const TileLayer *touchedLayer)
 {
     mError.clear();
     mWarning.clear();
+
     if (!mMapDocument)
         return;
 
     const bool automatic = touchedLayer != nullptr;
 
     if (!mLoaded) {
+        if (mRulesFile.isEmpty()) {
+            mError = tr("No AutoMapping rules provided. Save the map or refer to a rule file in the project properties.");
+            emit errorsOccurred(automatic);
+            return;
+        }
+
         if (loadFile(mRulesFile)) {
             mLoaded = true;
         } else {
@@ -131,13 +136,14 @@ void AutomappingManager::autoMapInternal(const QRegion &where,
 
         QUndoStack *undoStack = mMapDocument->undoStack();
         undoStack->beginMacro(tr("Apply AutoMap rules"));
-        AutoMapperWrapper *aw = new AutoMapperWrapper(mMapDocument, std::move(passedAutoMappers), &region);
+        AutoMapperWrapper *aw = new AutoMapperWrapper(mMapDocument, passedAutoMappers, &region);
         undoStack->push(aw);
         undoStack->endMacro();
     }
-    for (auto &automapper : qAsConst(mAutoMappers)) {
-        mWarning += automapper->warningString();
-        mError += automapper->errorString();
+
+    for (auto &autoMapper : qAsConst(passedAutoMappers)) {
+        mWarning += autoMapper->warningString();
+        mError += autoMapper->errorString();
     }
 
     if (!mWarning.isEmpty())
@@ -147,7 +153,24 @@ void AutomappingManager::autoMapInternal(const QRegion &where,
         emit errorsOccurred(automatic);
 }
 
+/**
+ * This function parses a rules file or loads a rules map file.
+ *
+ * While parsing a rules file, any listed files with extension "txt" will also
+ * be parsed as a rules file. Any listed file that loads as a map results in
+ * the creation of an AutoMapper instance.
+ *
+ * @return whether the loading was successful
+ */
 bool AutomappingManager::loadFile(const QString &filePath)
+{
+    if (filePath.endsWith(QLatin1String(".txt"), Qt::CaseInsensitive))
+        return loadRulesFile(filePath);
+
+    return loadRuleMap(filePath);
+}
+
+bool AutomappingManager::loadRulesFile(const QString &filePath)
 {
     bool ret = true;
     const QDir absPath = QFileInfo(filePath).dir();
@@ -173,23 +196,17 @@ bool AutomappingManager::loadFile(const QString &filePath)
     mWatcher.addPath(filePath);
 
     QTextStream in(&rulesFile);
-    QString line = in.readLine();
 
-    for (; !line.isNull(); line = in.readLine()) {
+    for (QString line = in.readLine(); !line.isNull(); line = in.readLine()) {
         QString rulePath = line.trimmed();
         if (rulePath.isEmpty()
                 || rulePath.startsWith(QLatin1Char('#'))
                 || rulePath.startsWith(QLatin1String("//")))
             continue;
 
-        QFileInfo rulePathInfo(rulePath);
+        rulePath = absPath.filePath(rulePath);
 
-        if (rulePathInfo.isRelative()) {
-            rulePath = absPath.filePath(rulePath);
-            rulePathInfo.setFile(rulePath);
-        }
-
-        if (!rulePathInfo.exists()) {
+        if (!QFileInfo::exists(rulePath)) {
             QString error = tr("File not found: '%1' (referenced by '%2')")
                     .arg(rulePath, filePath);
             ERROR(error);
@@ -199,43 +216,49 @@ bool AutomappingManager::loadFile(const QString &filePath)
             ret = false;
             continue;
         }
-        if (rulePath.endsWith(QLatin1String(".tmx"), Qt::CaseInsensitive)) {
-            QString errorString;
-            std::unique_ptr<Map> rules { readMap(rulePath, &errorString) };
 
-            if (!rules) {
-                QString error = tr("Opening rules map '%1' failed: %2")
-                        .arg(rulePath, errorString);
-                ERROR(error);
-
-                mError += error;
-                mError += QLatin1Char('\n');
-                ret = false;
-                continue;
-            }
-
-            std::unique_ptr<AutoMapper> autoMapper { new AutoMapper(mMapDocument, std::move(rules), rulePath) };
-
-            mWarning += autoMapper->warningString();
-            const QString error = autoMapper->errorString();
-            if (error.isEmpty()) {
-                mAutoMappers.push_back(std::move(autoMapper));
-                mWatcher.addPath(rulePath);
-            } else {
-                mError += error;
-            }
-        }
-        if (rulePath.endsWith(QLatin1String(".txt"), Qt::CaseInsensitive)) {
-            if (!loadFile(rulePath))
-                ret = false;
-        }
+        if (!loadFile(rulePath))
+            ret = false;
     }
+
     return ret;
 }
 
+bool AutomappingManager::loadRuleMap(const QString &filePath)
+{
+    QString errorString;
+    std::unique_ptr<Map> rules { readMap(filePath, &errorString) };
+
+    if (!rules) {
+        QString error = tr("Opening rules map '%1' failed: %2")
+                .arg(filePath, errorString);
+        ERROR(error);
+
+        mError += error;
+        mError += QLatin1Char('\n');
+        return false;
+    }
+
+    std::unique_ptr<AutoMapper> autoMapper { new AutoMapper(mMapDocument, std::move(rules), filePath) };
+
+    mWarning += autoMapper->warningString();
+    const QString error = autoMapper->errorString();
+    if (error.isEmpty()) {
+        mAutoMappers.push_back(std::move(autoMapper));
+        mWatcher.addPath(filePath);
+    } else {
+        mError += error;
+    }
+
+    return true;
+}
+
 /**
- * The rules file is determind based on the map location, but can be overridden
- * by passing \a rulesFile.
+ * The rules file is determined based on the map location, or taken from the
+ * current project if a "rules.txt" file does not exist alongside the map (or
+ * when the map is not saved).
+ *
+ * Alternatively, it can can be overridden by passing a non-empty \a rulesFile.
  */
 void AutomappingManager::setMapDocument(MapDocument *mapDocument, const QString &rulesFile)
 {
@@ -260,18 +283,25 @@ void AutomappingManager::setMapDocument(MapDocument *mapDocument, const QString
     refreshRulesFile(rulesFile);
 }
 
+/**
+ * Needs to be called when the project rules file path is changed.
+ *
+ * It is called automatically when the file name of the current MapDocument
+ * changes.
+ */
 void AutomappingManager::refreshRulesFile(const QString &ruleFileOverride)
 {
     mRulesFileOverride = !ruleFileOverride.isEmpty();
     QString rulesFile = ruleFileOverride;
 
-    if (rulesFile.isEmpty() && mMapDocument) {
+    if (rulesFile.isEmpty() && mMapDocument && !mMapDocument->fileName().isEmpty()) {
         const QString mapPath = QFileInfo(mMapDocument->fileName()).path();
         rulesFile = mapPath + QLatin1String("/rules.txt");
 
         if (!QFileInfo::exists(rulesFile)) {
             auto &project = ProjectManager::instance()->project();
-            rulesFile = project.mAutomappingRulesFile;
+            if (!project.mAutomappingRulesFile.isEmpty())
+                rulesFile = project.mAutomappingRulesFile;
         }
     }
 
@@ -293,3 +323,5 @@ void AutomappingManager::onFileChanged()
 {
     cleanUp();
 }
+
+#include "moc_automappingmanager.cpp"
