@@ -32,7 +32,7 @@
 
 namespace Tiled {
 
-class Layer;
+class TileLayer;
 
 class AutoMapper;
 class MapDocument;
@@ -48,12 +48,13 @@ class AutomappingManager : public QObject
 
 public:
     AutomappingManager(QObject *parent = nullptr);
-    ~AutomappingManager();
+    ~AutomappingManager() override;
 
     void setMapDocument(MapDocument *mapDocument, const QString &rulesFile = QString());
 
-    QString errorString() const { return mError; }
+    void refreshRulesFile(const QString &ruleFileOverride = QString());
 
+    QString errorString() const { return mError; }
     QString warningString() const { return mWarning; }
 
     /**
@@ -77,32 +78,21 @@ signals:
     void warningsOccurred(bool automatic);
 
 private:
-    void onRegionEdited(const QRegion &where, Layer *touchedLayer);
+    void onRegionEdited(const QRegion &where, TileLayer *touchedLayer);
     void onMapFileNameChanged();
     void onFileChanged();
 
-    void refreshRulesFile(const QString &ruleFileOverride = QString());
-
-    /**
-     * This function parses a rules file.
-     * For each path which is a rule, (file extension is tmx) an AutoMapper
-     * object is setup.
-     *
-     * If a file extension is txt, this file will be opened and searched for
-     * rules again.
-     *
-     * @return if the loading was successful: return true if it succeeded.
-     */
     bool loadFile(const QString &filePath);
+    bool loadRulesFile(const QString &filePath);
+    bool loadRuleMap(const QString &filePath);
 
     /**
-     * Applies automapping to the Region \a where, considering only layer
-     * \a touchedLayer has changed.
-     * There will only those Automappers be used which have a rule layer
-     * touching the \a touchedLayer
-     * If layer is 0, all Automappers are used.
+     * Applies automapping to the region \a where.
+     *
+     * If a \a touchedLayer is given, only those AutoMappers will be used which
+     * have a rule layer matching the \a touchedLayer.
      */
-    void autoMapInternal(const QRegion &where, Layer *touchedLayer);
+    void autoMapInternal(const QRegion &where, const TileLayer *touchedLayer);
 
     /**
      * deletes all its data structures
@@ -112,7 +102,7 @@ private:
     /**
      * The current map document.
      */
-    MapDocument *mMapDocument;
+    MapDocument *mMapDocument = nullptr;
 
     /**
      * For each new file of rules a new AutoMapper is setup. In this vector we
@@ -124,7 +114,7 @@ private:
      * This tells you if the rules for the current map document were already
      * loaded.
      */
-    bool mLoaded;
+    bool mLoaded = false;
 
     /**
      * Contains all errors which occurred until canceling.
