@@ -56,11 +56,22 @@ ChangeMapObject::ChangeMapObject(Document *document,
     }
 }
 
+bool ChangeMapObject::mergeWith(const QUndoCommand *other)
+{
+    auto o = static_cast<const ChangeMapObject*>(other);
+    if (mDocument == o->mDocument && mMapObject == o->mMapObject && mProperty == o->mProperty) {
+#if QT_VERSION >= QT_VERSION_CHECK(5, 9, 0)
+        setObsolete(mMapObject->mapObjectProperty(mProperty) == mValue);
+#endif
+        return true;
+    }
+    return false;
+}
+
 void ChangeMapObject::swap()
 {
-    QVariant oldValue = mMapObject->mapObjectProperty(mProperty);
-    mMapObject->setMapObjectProperty(mProperty, mValue);
-    std::swap(mValue, oldValue);
+    const auto value = std::exchange(mValue, mMapObject->mapObjectProperty(mProperty));
+    mMapObject->setMapObjectProperty(mProperty, value);
 
     mMapObject->setPropertyChanged(mProperty, mNewChangeState);
     std::swap(mOldChangeState, mNewChangeState);
