@@ -104,8 +104,6 @@ CommandDataModel *CommandTreeView::model() const
 void CommandTreeView::contextMenuEvent(QContextMenuEvent *event)
 {
     const QModelIndex index = indexAt(event->pos());
-
-    // Generate a run a menu for the index
     if (QMenu *menu = model()->contextMenu(this, index))
         menu->exec(event->globalPos());
 }
@@ -133,3 +131,5 @@ void CommandTreeView::removeSelectedCommands()
     const QModelIndexList indices = selectionModel()->selectedRows();
     model()->removeRows(indices);
 }
+
+#include "moc_commanddialog.cpp"
