@@ -20,7 +20,7 @@
 
 #include "editableimagelayer.h"
 
-#include "changeimagelayerproperties.h"
+#include "changeimagelayerproperty.h"
 #include "editablemap.h"
 #include "scriptimage.h"
 
@@ -39,11 +39,10 @@ EditableImageLayer::EditableImageLayer(EditableMap *map, ImageLayer *imageLayer,
 void EditableImageLayer::setTransparentColor(const QColor &transparentColor)
 {
     if (auto doc = mapDocument()) {
-        asset()->push(new ChangeImageLayerProperties(doc,
-                                                     imageLayer(),
-                                                     transparentColor,
-                                                     imageSource()));
-    } else {
+        asset()->push(new ChangeImageLayerTransparentColor(doc,
+                                                           { imageLayer() },
+                                                           transparentColor));
+    } else if (!checkReadOnly()) {
         imageLayer()->setTransparentColor(transparentColor);
         if (!imageSource().isEmpty())
             imageLayer()->loadFromImage(imageSource());
@@ -53,11 +52,10 @@ void EditableImageLayer::setTransparentColor(const QColor &transparentColor)
 void EditableImageLayer::setImageSource(const QUrl &imageSource)
 {
     if (auto doc = mapDocument()) {
-        asset()->push(new ChangeImageLayerProperties(doc,
-                                                     imageLayer(),
-                                                     transparentColor(),
-                                                     imageSource));
-    } else {
+        asset()->push(new ChangeImageLayerImageSource(doc,
+                                                      { imageLayer() },
+                                                      imageSource));
+    } else if (!checkReadOnly()) {
         if (imageSource.isEmpty())
             imageLayer()->resetImage();
         else
@@ -67,10 +65,34 @@ void EditableImageLayer::setImageSource(const QUrl &imageSource)
 
 void EditableImageLayer::setImage(ScriptImage *image, const QUrl &source)
 {
+    if (checkReadOnly())
+        return;
+
     // WARNING: This function has no undo!
     imageLayer()->loadFromImage(QPixmap::fromImage(image->image()), source);
 }
 
+void EditableImageLayer::setRepeatX(bool repeatX)
+{
+    if (auto doc = mapDocument()) {
+        asset()->push(new ChangeImageLayerRepeatX(doc,
+                                                  { imageLayer() },
+                                                  repeatX));
+    } else if (!checkReadOnly()) {
+        imageLayer()->setRepeatX(repeatX);
+    }
+}
+
+void EditableImageLayer::setRepeatY(bool repeatY)
+{
+    if (auto doc = mapDocument()) {
+        asset()->push(new ChangeImageLayerRepeatY(doc,
+                                                  { imageLayer() },
+                                                  repeatY));
+    } else if (!checkReadOnly()) {
+        imageLayer()->setRepeatY(repeatY);
+    }
+}
 } // namespace Tiled
 
 #include "moc_editableimagelayer.cpp"
