@@ -27,6 +27,7 @@
 
 namespace Tiled {
 
+class EditableGroupLayer;
 class EditableMap;
 class MapDocument;
 
@@ -34,12 +35,16 @@ class EditableLayer : public EditableObject
 {
     Q_OBJECT
 
+    Q_PROPERTY(int id READ id)
     Q_PROPERTY(QString name READ name WRITE setName)
     Q_PROPERTY(qreal opacity READ opacity WRITE setOpacity)
+    Q_PROPERTY(QColor tintColor READ tintColor WRITE setTintColor)
     Q_PROPERTY(bool visible READ isVisible WRITE setVisible)
     Q_PROPERTY(bool locked READ isLocked WRITE setLocked)
     Q_PROPERTY(QPointF offset READ offset WRITE setOffset)
+    Q_PROPERTY(QPointF parallaxFactor READ parallaxFactor WRITE setParallaxFactor)
     Q_PROPERTY(Tiled::EditableMap *map READ map)
+    Q_PROPERTY(Tiled::EditableGroupLayer *parentLayer READ parentLayer)
     Q_PROPERTY(bool selected READ isSelected WRITE setSelected)
     Q_PROPERTY(bool isTileLayer READ isTileLayer CONSTANT)
     Q_PROPERTY(bool isObjectLayer READ isObjectLayer CONSTANT)
@@ -55,12 +60,16 @@ public:
                   QObject *parent = nullptr);
     ~EditableLayer() override;
 
+    int id() const;
     const QString &name() const;
     qreal opacity() const;
+    QColor tintColor() const;
     bool isVisible() const;
     bool isLocked() const;
     QPointF offset() const;
+    QPointF parallaxFactor() const;
     EditableMap *map() const;
+    EditableGroupLayer *parentLayer() const;
     bool isSelected() const;
     bool isTileLayer() const;
     bool isObjectLayer() const;
@@ -78,9 +87,11 @@ public:
 public slots:
     void setName(const QString &name);
     void setOpacity(qreal opacity);
+    void setTintColor(const QColor &color);
     void setVisible(bool visible);
     void setLocked(bool locked);
     void setOffset(QPointF offset);
+    void setParallaxFactor(QPointF factor);
     void setSelected(bool selected);
 
 protected:
@@ -91,6 +102,11 @@ private:
 };
 
 
+inline int EditableLayer::id() const
+{
+    return layer()->id();
+}
+
 inline const QString &EditableLayer::name() const
 {
     return layer()->name();
@@ -101,6 +117,11 @@ inline qreal EditableLayer::opacity() const
     return layer()->opacity();
 }
 
+inline QColor EditableLayer::tintColor() const
+{
+    return layer()->tintColor();
+}
+
 inline bool EditableLayer::isVisible() const
 {
     return layer()->isVisible();
@@ -116,6 +137,11 @@ inline QPointF EditableLayer::offset() const
     return layer()->offset();
 }
 
+inline QPointF EditableLayer::parallaxFactor() const
+{
+    return layer()->parallaxFactor();
+}
+
 inline bool EditableLayer::isTileLayer() const
 {
     return layer()->isTileLayer();
