@@ -21,6 +21,7 @@
 #include "editablemap.h"
 
 #include "addremovelayer.h"
+#include "addremovemapobject.h"
 #include "addremovetileset.h"
 #include "automappingmanager.h"
 #include "changeevents.h"
@@ -47,6 +48,7 @@
 #include "tilesetdocument.h"
 
 #include <QCoreApplication>
+#include <QQmlEngine>
 #include <QUndoStack>
 
 #include "qtcompat_p.h"
@@ -55,15 +57,12 @@ namespace Tiled {
 
 EditableMap::EditableMap(QObject *parent)
     : EditableAsset(nullptr, new Map(), parent)
-    , mReadOnly(false)
-    , mSelectedArea(nullptr)
 {
     mDetachedMap.reset(map());
 }
 
 EditableMap::EditableMap(MapDocument *mapDocument, QObject *parent)
     : EditableAsset(mapDocument, mapDocument->map(), parent)
-    , mReadOnly(false)
     , mSelectedArea(new EditableSelectedArea(mapDocument, this))
 {
     connect(mapDocument, &Document::fileNameChanged, this, &EditableAsset::fileNameChanged);
@@ -71,9 +70,11 @@ EditableMap::EditableMap(MapDocument *mapDocument, QObject *parent)
     connect(mapDocument, &MapDocument::layerAdded, this, &EditableMap::attachLayer);
     connect(mapDocument, &MapDocument::layerRemoved, this, &EditableMap::detachLayer);
 
-    connect(mapDocument, &MapDocument::currentLayerChanged, this, &EditableMap::onCurrentLayerChanged);
+    connect(mapDocument, &MapDocument::currentLayerChanged, this, &EditableMap::currentLayerChanged);
     connect(mapDocument, &MapDocument::selectedLayersChanged, this, &EditableMap::selectedLayersChanged);
     connect(mapDocument, &MapDocument::selectedObjectsChanged, this, &EditableMap::selectedObjectsChanged);
+
+    connect(mapDocument, &MapDocument::regionEdited, this, &EditableMap::onRegionEdited);
 }
 
 /**
@@ -91,8 +92,6 @@ EditableMap::EditableMap(const Map *map, QObject *parent)
 EditableMap::EditableMap(std::unique_ptr<Map> map, QObject *parent)
     : EditableAsset(nullptr, map.get(), parent)
     , mDetachedMap(std::move(map))
-    , mReadOnly(false)
-    , mSelectedArea(nullptr)
 {
 }
 
@@ -107,15 +106,23 @@ QList<QObject *> EditableMap::tilesets() const
     QList<QObject *> editableTilesets;
     auto &editableManager = EditableManager::instance();
 
-    for (const SharedTileset &tileset : map()->tilesets()) {
-        if (auto document = TilesetDocument::findDocumentForTileset(tileset))
-            editableTilesets.append(document->editable());
-        else
-            editableTilesets.append(editableManager.editableTileset(tileset.data()));
-    }
+    for (const SharedTileset &tileset : map()->tilesets())
+        editableTilesets.append(editableManager.editableTileset(tileset.data()));
+
     return editableTilesets;
 }
 
+QList<QObject *> EditableMap::layers()
+{
+    QList<QObject *> editables;
+    auto &editableManager = EditableManager::instance();
+
+    for (const auto layer : map()->layers())
+        editables.append(editableManager.editableLayer(this, layer));
+
+    return editables;
+}
+
 EditableLayer *EditableMap::currentLayer()
 {
     if (auto document = mapDocument())
@@ -131,7 +138,8 @@ QList<QObject *> EditableMap::selectedLayers()
     QList<QObject*> selectedLayers;
 
     auto &editableManager = EditableManager::instance();
-    for (Layer *layer : mapDocument()->selectedLayers())
+    const auto selectedLayersOrdered = mapDocument()->selectedLayersOrdered();
+    for (Layer *layer : selectedLayersOrdered)
         selectedLayers.append(editableManager.editableLayer(this, layer));
 
     return selectedLayers;
@@ -145,7 +153,8 @@ QList<QObject *> EditableMap::selectedObjects()
     QList<QObject*> selectedObjects;
 
     auto &editableManager = EditableManager::instance();
-    for (MapObject *object : mapDocument()->selectedObjects())
+    const auto selectedObjectsOrdered = mapDocument()->selectedObjectsOrdered();
+    for (MapObject *object : selectedObjectsOrdered)
         selectedObjects.append(editableManager.editableMapObject(this, object));
 
     return selectedObjects;
@@ -237,7 +246,7 @@ bool EditableMap::addTileset(EditableTileset *editableTileset)
         ScriptManager::instance().throwNullArgError(0);
         return false;
     }
-    const auto &tileset = editableTileset->tileset()->sharedPointer();
+    const auto &tileset = editableTileset->tileset()->sharedFromThis();
     if (map()->indexOfTileset(tileset) != -1)
         return false;   // can't add existing tileset
 
@@ -265,12 +274,12 @@ bool EditableMap::replaceTileset(EditableTileset *oldEditableTileset,
         return false;
     }
 
-    SharedTileset oldTileset = oldEditableTileset->tileset()->sharedPointer();
+    SharedTileset oldTileset = oldEditableTileset->tileset()->sharedFromThis();
     int indexOfOldTileset = map()->indexOfTileset(oldTileset);
     if (indexOfOldTileset == -1)
         return false;   // can't replace non-existing tileset
 
-    SharedTileset newTileset = newEditableTileset->tileset()->sharedPointer();
+    SharedTileset newTileset = newEditableTileset->tileset()->sharedFromThis();
     int indexOfNewTileset = map()->indexOfTileset(newTileset);
     if (indexOfNewTileset != -1)
         return false;   // can't replace with tileset that is already part of the map (undo broken)
@@ -290,7 +299,7 @@ bool EditableMap::removeTileset(EditableTileset *editableTileset)
         return false;
     }
     Tileset *tileset = editableTileset->tileset();
-    int index = map()->indexOfTileset(tileset->sharedPointer());
+    int index = map()->indexOfTileset(tileset->sharedFromThis());
     if (index == -1)
         return false;   // can't remove non-existing tileset
 
@@ -311,11 +320,41 @@ QList<QObject *> EditableMap::usedTilesets() const
 
     QList<QObject *> editableTilesets;
     for (const SharedTileset &tileset : tilesets)
-        if (auto document = TilesetDocument::findDocumentForTileset(tileset))
-            editableTilesets.append(document->editable());
+        editableTilesets.append(EditableManager::instance().editableTileset(tileset.data()));
     return editableTilesets;
 }
 
+void EditableMap::removeObjects(const QList<QObject*> &objects)
+{
+    QList<MapObject *> mapObjects;
+    mapObjects.reserve(objects.size());
+
+    for (QObject *object : objects) {
+        auto editableMapObject = qobject_cast<EditableMapObject*>(object);
+        if (!editableMapObject) {
+            ScriptManager::instance().throwError(QCoreApplication::translate("Script Errors", "Not an object"));
+            return;
+        }
+        if (editableMapObject->map() != this) {
+            ScriptManager::instance().throwError(QCoreApplication::translate("Script Errors", "Object not from this map"));
+            return;
+        }
+
+        auto mapObject = editableMapObject->mapObject();
+        if (!mapObjects.contains(mapObject))
+            mapObjects.append(mapObject);
+    }
+
+    if (auto doc = mapDocument()) {
+        asset()->push(new RemoveMapObjects(doc, mapObjects));
+    } else {
+        for (MapObject *mapObject : mapObjects) {
+            mapObject->objectGroup()->removeObject(mapObject);
+            EditableManager::instance().release(mapObject);
+        }
+    }
+}
+
 /**
  * Merges the given map with this map. Automatically adds any tilesets that are
  * used by the merged map which are not yet part of this map.
@@ -347,8 +386,8 @@ void EditableMap::merge(EditableMap *editableMap, bool canJoin)
     }
 
     QVector<SharedTileset> missingTilesets;
-    mapDocument()->unifyTilesets(map, missingTilesets);
-    mapDocument()->paintTileLayers(map, canJoin, &missingTilesets);
+    mapDocument()->unifyTilesets(*map, missingTilesets);
+    mapDocument()->paintTileLayers(*map, canJoin, &missingTilesets);
 }
 
 /**
@@ -498,6 +537,14 @@ void EditableMap::setStaggerIndex(StaggerIndex value)
         map()->setStaggerIndex(static_cast<Map::StaggerIndex>(value));
 }
 
+void EditableMap::setParallaxOrigin(const QPointF &parallaxOrigin)
+{
+    if (auto doc = mapDocument())
+        push(new ChangeMapProperty(doc, parallaxOrigin));
+    else if (!checkReadOnly())
+        map()->setParallaxOrigin(parallaxOrigin);
+}
+
 void EditableMap::setOrientation(Orientation value)
 {
     if (auto doc = mapDocument()) {
@@ -591,6 +638,18 @@ void EditableMap::setSelectedObjects(const QList<QObject *> &objects)
     document->setSelectedObjects(plainObjects);
 }
 
+QSharedPointer<Document> EditableMap::createDocument()
+{
+    Q_ASSERT(mDetachedMap);
+
+    auto document = MapDocumentPtr::create(std::move(mDetachedMap));
+    document->setEditable(std::unique_ptr<EditableAsset>(this));
+
+    QQmlEngine::setObjectOwnership(this, QQmlEngine::CppOwnership);
+
+    return document;
+}
+
 void EditableMap::documentChanged(const ChangeEvent &change)
 {
     switch (change.type) {
@@ -656,9 +715,11 @@ void EditableMap::detachMapObjects(const QList<MapObject *> &mapObjects)
     }
 }
 
-void EditableMap::onCurrentLayerChanged(Layer *)
+void EditableMap::onRegionEdited(const QRegion &region, TileLayer *layer)
 {
-    emit currentLayerChanged();
+    auto &editableManager = EditableManager::instance();
+    const auto editableLayer = static_cast<EditableTileLayer*>(editableManager.editableLayer(this, layer));
+    emit regionEdited(RegionValueType(region), editableLayer);
 }
 
 MapRenderer *EditableMap::renderer() const
