@@ -73,7 +73,7 @@ void EditableObjectGroup::removeObjectAt(int index)
 
     if (auto doc = document()) {
         asset()->push(new RemoveMapObjects(doc, mapObject));
-    } else {
+    } else if (!checkReadOnly()) {
         objectGroup()->removeObjectAt(index);
         EditableManager::instance().release(mapObject);
     }
@@ -105,17 +105,28 @@ void EditableObjectGroup::insertObjectAt(int index, EditableMapObject *editableM
         return;
     }
 
-    if (editableMapObject->mapObject()->objectGroup()) {
+    auto mapObject = editableMapObject->mapObject();
+
+    if (mapObject->objectGroup()) {
         ScriptManager::instance().throwError(QCoreApplication::translate("Script Errors", "Object already part of an object layer"));
         return;
     }
 
+    if (checkReadOnly())
+        return;
+
+    // Avoid duplicate IDs by resetting when needed
+    if (Map *map = objectGroup()->map()) {
+        if (mapObject->id() != 0 && map->findObjectById(mapObject->id()))
+            mapObject->resetId();
+    }
+
     if (auto doc = document()) {
-        AddRemoveMapObjects::Entry entry { editableMapObject->mapObject(), objectGroup() };
+        AddRemoveMapObjects::Entry entry { mapObject, objectGroup() };
         entry.index = index;
         asset()->push(new AddMapObjects(doc, { entry }));
     } else {
-        objectGroup()->insertObject(index, editableMapObject->mapObject());
+        objectGroup()->insertObject(index, mapObject);
         editableMapObject->release();   // now owned by the object group
     }
 }
@@ -128,13 +139,23 @@ void EditableObjectGroup::addObject(EditableMapObject *editableMapObject)
 void EditableObjectGroup::setColor(const QColor &color)
 {
     if (auto doc = document()) {
-        asset()->push(new ChangeObjectGroupProperties(doc,
-                                                      objectGroup(),
-                                                      color,
-                                                      objectGroup()->drawOrder()));
-    } else {
+        asset()->push(new ChangeObjectGroupColor(doc, { objectGroup() }, color));
+    } else if (!checkReadOnly()) {
         objectGroup()->setColor(color);
     }
 }
 
+void EditableObjectGroup::setDrawOrder(DrawOrder drawOrder)
+{
+    if (auto doc = document()) {
+        asset()->push(new ChangeObjectGroupDrawOrder(doc,
+                                                     { objectGroup() },
+                                                     static_cast<ObjectGroup::DrawOrder>(drawOrder)));
+    } else if (!checkReadOnly()) {
+        objectGroup()->setDrawOrder(static_cast<ObjectGroup::DrawOrder>(drawOrder));
+    }
+}
+
 } // namespace Tiled
+
+#include "moc_editableobjectgroup.cpp"
