@@ -109,6 +109,8 @@ void Eraser::doErase(bool continuation)
     }
     mLastTilePos = tilePos;
 
+    QList<QPair<QRegion, TileLayer*>> erasedRegions;
+
     auto eraseOnLayer = [&] (TileLayer *tileLayer) {
         if (!tileLayer->isUnlocked())
             return;
@@ -121,7 +123,7 @@ void Eraser::doErase(bool continuation)
         erase->setMergeable(continuation);
 
         mapDocument()->undoStack()->push(erase);
-        emit mapDocument()->regionEdited(eraseRegion, tileLayer);
+        erasedRegions.append({ eraseRegion, tileLayer });
 
         continuation = true;    // further erases are always continuations
     };
@@ -129,10 +131,19 @@ void Eraser::doErase(bool continuation)
     if (mAllLayers) {
         for (Layer *layer : mapDocument()->map()->tileLayers())
             eraseOnLayer(static_cast<TileLayer*>(layer));
-    } else {
+    } else if (!mapDocument()->selectedLayers().isEmpty()) {
         for (Layer *layer : mapDocument()->selectedLayers())
             if (TileLayer *tileLayer = layer->asTileLayer())
                 eraseOnLayer(tileLayer);
+    } else if (auto tileLayer = currentTileLayer()) {
+        eraseOnLayer(tileLayer);
+    }
+
+    for (auto &erased : qAsConst(erasedRegions)) {
+        if (erased.second->map() != mapDocument()->map())
+            continue;
+
+        emit mapDocument()->regionEdited(erased.first, erased.second);
     }
 }
 
