@@ -37,6 +37,8 @@ public:
     void mousePressed(QGraphicsSceneMouseEvent *event) override;
     void mouseReleased(QGraphicsSceneMouseEvent *event) override;
 
+    void modifiersChanged(Qt::KeyboardModifiers) override;
+
     void languageChanged() override;
 
 protected:
@@ -52,7 +54,8 @@ private:
         RectangleErase
     };
 
-    Mode mMode;
+    Mode mMode = Nothing;
+    bool mAllLayers = false;
     QPoint mLastTilePos;
     QPoint mStart;
 };
