@@ -455,9 +455,9 @@ void LayerView::keyPressEvent(QKeyEvent *event)
         if (layer) {
             QUndoCommand *command = nullptr;
             if (event->modifiers() & Qt::ControlModifier)
-                command = new SetLayerLocked(mMapDocument, layer, !layer->isLocked());
+                command = new SetLayerLocked(mMapDocument, { layer }, !layer->isLocked());
             else
-                command = new SetLayerVisible(mMapDocument, layer, !layer->isVisible());
+                command = new SetLayerVisible(mMapDocument, { layer }, !layer->isVisible());
             mMapDocument->undoStack()->push(command);
             return;
         }
@@ -489,5 +489,14 @@ void LayerView::selectionChanged(const QItemSelection &selected,
     mMapDocument->setSelectedLayers(layers);
 }
 
+QItemSelectionModel::SelectionFlags LayerView::selectionCommand(const QModelIndex &index, const QEvent *event) const
+{
+    // Prevent deselect by clicking on empty area
+    if (!index.isValid() && event && event->type() == QEvent::MouseButtonRelease)
+        return QItemSelectionModel::NoUpdate;
+
+    return QTreeView::selectionCommand(index, event);
+}
+
 #include "layerdock.moc"
 #include "moc_layerdock.cpp"
