@@ -57,6 +57,7 @@
 #include "newsbutton.h"
 #include "newtilesetdialog.h"
 #include "objectgroup.h"
+#include "propertytypeseditor.h"
 #include "objecttypeseditor.h"
 #include "offsetmapdialog.h"
 #include "projectdock.h"
@@ -104,8 +105,13 @@
 #include <QVariantAnimation>
 
 #ifdef Q_OS_WIN
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+#include <QWindow>
+#include <QtGui/qpa/qplatformwindow_p.h>
+#else
 #include <QtPlatformHeaders\QWindowsWindowFunctions>
 #endif
+#endif // Q_OS_WIN
 
 #include "locatorwidget.h"
 #include "qtcompat_p.h"
@@ -232,6 +238,7 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
     , mUi(new Ui::MainWindow)
     , mActionHandler(new MapDocumentActionHandler(this))
     , mObjectTypesEditor(new ObjectTypesEditor(this))
+    , mPropertyTypesEditor(new PropertyTypesEditor(this))
     , mAutomappingManager(new AutomappingManager(this))
     , mDocumentManager(nullptr)
 {
@@ -355,7 +362,6 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
     redoAction->setPriority(QAction::LowPriority);
     redoAction->setIcon(redoIcon);
     undoAction->setIcon(undoIcon);
-    connect(undoGroup, &QUndoGroup::cleanChanged, this, &MainWindow::updateWindowTitle);
 
     mUi->actionNewMap->setShortcuts(QKeySequence::New);
     mUi->actionOpen->setShortcuts(QKeySequence::Open);
@@ -514,6 +520,7 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
     mLayerMenu->addSeparator();
     mLayerMenu->addAction(mActionHandler->actionSelectPreviousLayer());
     mLayerMenu->addAction(mActionHandler->actionSelectNextLayer());
+    mLayerMenu->addAction(mActionHandler->actionSelectAllLayers());
     mLayerMenu->addAction(mActionHandler->actionMoveLayersUp());
     mLayerMenu->addAction(mActionHandler->actionMoveLayersDown());
     mLayerMenu->addSeparator();
@@ -590,7 +597,8 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
     connect(mUi->actionLoadWorld, &QAction::triggered, this, [this] {
         Session &session = Session::current();
         QString lastPath = session.lastPath(Session::WorldFile);
-        QString filter = tr("All Files (*);;");
+        QString filter = tr("All Files (*)");
+        filter.append(QStringLiteral(";;"));
         QString worldFilesFilter = tr("World files (*.world)");
         filter.append(worldFilesFilter);
         QString worldFile = QFileDialog::getOpenFileName(this, tr("Load World"), lastPath,
@@ -625,7 +633,8 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
     connect(mUi->actionNewWorld, &QAction::triggered, this, [this] {
         Session &session = Session::current();
         QString lastPath = session.lastPath(Session::WorldFile);
-        QString filter = tr("All Files (*);;");
+        QString filter = tr("All Files (*)");
+        filter.append(QStringLiteral(";;"));
         QString worldFilesFilter = tr("World files (*.world)");
         filter.append(worldFilesFilter);
         QString worldFile;
@@ -794,8 +803,13 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
 
     mShowObjectTypesEditor = new QAction(tr("Object Types Editor"), this);
     mShowObjectTypesEditor->setCheckable(true);
+
+    mShowPropertyTypesEditor = new QAction(tr("Property Types Editor"), this);
+    mShowPropertyTypesEditor->setCheckable(true);
+
     mUi->menuView->insertAction(mUi->actionShowGrid, mViewsAndToolbarsAction);
     mUi->menuView->insertAction(mUi->actionShowGrid, mShowObjectTypesEditor);
+    mUi->menuView->insertAction(mUi->actionShowGrid, mShowPropertyTypesEditor);
     mUi->menuView->insertSeparator(mUi->actionShowGrid);
 
     mUi->menuTileset->insertAction(mUi->actionTilesetProperties, mTilesetEditor->showAnimationEditor());
@@ -815,6 +829,11 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
     connect(mObjectTypesEditor, &ObjectTypesEditor::closed,
             this, &MainWindow::onObjectTypesEditorClosed);
 
+    connect(mShowPropertyTypesEditor, &QAction::toggled,
+            mPropertyTypesEditor, &QWidget::setVisible);
+    connect(mPropertyTypesEditor, &PropertyTypesEditor::closed,
+            this, &MainWindow::onPropertyTypesEditorClosed);
+
     connect(ClipboardManager::instance(), &ClipboardManager::hasMapChanged,
             this, &MainWindow::updateActions);
 
@@ -1008,9 +1027,6 @@ void MainWindow::newMap()
 
     emit mDocumentManager->documentCreated(mapDocument.data());
 
-    if (!mDocumentManager->saveDocumentAs(mapDocument.data()))
-        return;
-
     mDocumentManager->addDocument(mapDocument);
 }
 
@@ -1023,7 +1039,6 @@ void MainWindow::initializeSession()
     bool projectLoaded = !session.project.isEmpty() && project.load(session.project);
 
     if (projectLoaded) {
-        Preferences::instance()->setObjectTypesFile(project.mObjectTypesFile);
         ProjectManager::instance()->setProject(std::move(project));
         updateWindowTitle();
         updateActions();
@@ -1208,7 +1223,6 @@ void MainWindow::saveAll()
             continue;
 
         QString fileName(document->fileName());
-        QString error;
 
         if (fileName.isEmpty()) {
             mDocumentManager->switchToDocument(document.data());
@@ -1507,7 +1521,6 @@ bool MainWindow::switchProject(Project project)
         prefs->addRecentProject(project.fileName());
     }
 
-    prefs->setObjectTypesFile(project.mObjectTypesFile);
     ProjectManager::instance()->setProject(std::move(project));
 
     restoreSession();
@@ -1546,6 +1559,7 @@ void MainWindow::projectProperties()
         Preferences::instance()->setObjectTypesFile(project.mObjectTypesFile);
 
         ScriptManager::instance().refreshExtensionsPaths();
+        mAutomappingManager->refreshRulesFile();
     }
 }
 
@@ -1600,6 +1614,7 @@ void MainWindow::openCrashReporterPopup()
     auto label = new QLabel;
     label->setTextFormat(Qt::RichText);
     label->setText(tr("<html>Enable anonymous crash reporting? <a href=\"https://www.mapeditor.org/crash-reporting\">more information</a></html>"));
+    label->setOpenExternalLinks(true);
 
     auto yesButton = new QPushButton(tr("&Yes"));
     auto noButton = new QPushButton(tr("&No"));
@@ -1615,9 +1630,6 @@ void MainWindow::openCrashReporterPopup()
     auto popup = new PopupWidget(this);
     popup->setLayout(layout);
 
-    connect(label, &QLabel::linkActivated, [] (const QString &link) {
-        QDesktopServices::openUrl(QUrl(link));
-    });
     connect(yesButton, &QPushButton::clicked, [] {
         Sentry::instance()->setUserConsent(Sentry::ConsentGiven);
     });
@@ -1641,6 +1653,7 @@ void MainWindow::openProjectExtensionsPopup()
     auto label = new QLabel;
     label->setTextFormat(Qt::RichText);
     label->setText(tr("The current project contains <a href=\"https://doc.mapeditor.org/en/stable/reference/scripting/\">scripted extensions</a>.<br><i>Make sure you trust those extensions before enabling them!</i>"));
+    label->setOpenExternalLinks(true);
 
     auto enableButton = new QPushButton(tr("&Enable Extensions"));
     auto closeButton = new QPushButton(tr("&Close"));
@@ -1655,9 +1668,6 @@ void MainWindow::openProjectExtensionsPopup()
     popup->setLayout(layout);
     popup->setTint(Qt::yellow);
 
-    connect(label, &QLabel::linkActivated, [] (const QString &link) {
-        QDesktopServices::openUrl(QUrl(link));
-    });
     connect(enableButton, &QPushButton::clicked, [popup] {
         ScriptManager::instance().enableProjectExtensions();
         popup->close();
@@ -1753,10 +1763,11 @@ void MainWindow::setFullScreen(bool fullScreen)
 
 void MainWindow::toggleClearView(bool clearView)
 {
-    MapView *mapView = nullptr;
+    MapEditor *mapEditor = nullptr;
+
     if (mDocument && mDocument->type() == Document::MapDocumentType) {
-        auto mapEditor = static_cast<MapEditor*>(mDocumentManager->editor(Document::MapDocumentType));
-        mapView = mapEditor->currentMapView();
+        mapEditor = static_cast<MapEditor*>(mDocumentManager->editor(Document::MapDocumentType));
+        MapView *mapView = mapEditor->currentMapView();
         mapView->setResizeAnchor(QGraphicsView::AnchorViewCenter);
     }
 
@@ -1788,8 +1799,12 @@ void MainWindow::toggleClearView(bool clearView)
 
     layout()->activate();
 
-    if (mapView)
+    if (mapEditor) {
+        mapEditor->editorWidget()->layout()->activate();
+
+        MapView *mapView = mapEditor->currentMapView();
         mapView->setResizeAnchor(QGraphicsView::NoAnchor);
+    }
 }
 
 void MainWindow::setLayoutLocked(bool locked)
@@ -2011,6 +2026,11 @@ void MainWindow::onObjectTypesEditorClosed()
     mShowObjectTypesEditor->setChecked(false);
 }
 
+void MainWindow::onPropertyTypesEditorClosed()
+{
+    mShowPropertyTypesEditor->setChecked(false);
+}
+
 void MainWindow::ensureHasBorderInFullScreen()
 {
 #ifdef Q_OS_WIN
@@ -2029,7 +2049,13 @@ void MainWindow::ensureHasBorderInFullScreen()
 
     bool wasFullScreen = isFullScreen();
     setFullScreen(false);
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     QWindowsWindowFunctions::setHasBorderInFullScreen(window, true);
+#else
+    using QWindowsWindow = QNativeInterface::Private::QWindowsWindow;
+    if (auto windowsWindow = window->nativeInterface<QWindowsWindow>())
+        windowsWindow->setHasBorderInFullScreen(true);
+#endif
     setFullScreen(wasFullScreen);
 
     hasBorderInFullScreen = true;
@@ -2176,7 +2202,7 @@ void MainWindow::updateActions()
     mUi->actionSaveAll->setEnabled(document);
 
     mUi->actionExportAsImage->setEnabled(mapDocument);
-    mUi->actionExport->setEnabled(mapDocument);
+    mUi->actionExport->setEnabled(mapDocument || tilesetDocument);
     mUi->actionExportAs->setEnabled(mapDocument || tilesetDocument);
     mUi->actionReload->setEnabled(mapDocument || (tilesetDocument && tilesetDocument->canReload()));
     mUi->actionClose->setEnabled(document);
@@ -2327,6 +2353,7 @@ void MainWindow::retranslateUi()
     mResetToDefaultLayout->setText(tr("Reset to Default Layout"));
     mLockLayout->setText(tr("Lock Layout"));
     mShowObjectTypesEditor->setText(tr("Object Types Editor"));
+    mShowPropertyTypesEditor->setText(tr("Property Types Editor"));
     mActionHandler->retranslateUi();
     CommandManager::instance()->retranslateUi();
 }
@@ -2448,6 +2475,8 @@ void MainWindow::documentChanged(Document *document)
     if (document) {
         connect(document, &Document::fileNameChanged,
                 this, &MainWindow::updateWindowTitle);
+        connect(document, &Document::modifiedChanged,
+                this, &MainWindow::updateWindowTitle);
 
         mProjectDock->selectFile(document->fileName());
     }
