@@ -234,12 +234,9 @@ QString MapDocument::displayName() const
     return displayName;
 }
 
-EditableAsset *MapDocument::editable()
+std::unique_ptr<EditableAsset> MapDocument::createEditable()
 {
-    if (!mEditable)
-        mEditable.reset(new EditableMap(this, this));
-
-    return mEditable.get();
+    return std::make_unique<EditableMap>(this, this);
 }
 
 /**
@@ -368,7 +365,7 @@ void MapDocument::resizeMap(QSize size, QPoint offset, bool removeObjects)
         case Layer::ImageLayerType: {
             // Adjust image layer by changing its offset
             auto imageLayer = static_cast<ImageLayer*>(layer);
-            new SetLayerOffset(this, layer,
+            new SetLayerOffset(this, { layer },
                                imageLayer->offset() + pixelOffset,
                                command);
             break;
@@ -777,17 +774,17 @@ void MapDocument::removeLayers(const QList<Layer *> &layers)
 /**
  * \see LayerModel::toggleLayers
  */
-void MapDocument::toggleLayers(const QList<Layer *> &layers)
+void MapDocument::toggleLayers(QList<Layer *> layers)
 {
-    mLayerModel->toggleLayers(layers);
+    mLayerModel->toggleLayers(std::move(layers));
 }
 
 /**
  * \see LayerModel::toggleLockLayers
  */
-void MapDocument::toggleLockLayers(const QList<Layer *> &layers)
+void MapDocument::toggleLockLayers(QList<Layer *> layers)
 {
-    mLayerModel->toggleLockLayers(layers);
+    mLayerModel->toggleLockLayers(std::move(layers));
 }
 
 /**
@@ -855,15 +852,77 @@ SharedTileset MapDocument::replaceTileset(int index, const SharedTileset &tilese
     return oldTileset;
 }
 
+QList<TileLayer *> MapDocument::findTargetLayers(const QList<const TileLayer *> &sourceLayers) const
+{
+    QList<TileLayer *> result;
+    result.reserve(sourceLayers.size());
+
+    // When the map contains only a single layer, always paint it into the
+    // current layer. This makes sure you can still take pieces from one layer
+    // and draw them into another.
+    if (sourceLayers.size() == 1 && mCurrentLayer && mCurrentLayer->isTileLayer()) {
+        result.append(static_cast<TileLayer*>(mCurrentLayer));
+        return result;
+    }
+
+    const int selectedTileLayerCount = std::count_if(selectedLayers().begin(),
+                                                     selectedLayers().end(),
+                                                     [] (const Layer *layer) { return layer->isTileLayer(); });
+
+    // If we have the same number of layers selected that we want to paint,
+    // these are the layers we will target.
+    if (selectedTileLayerCount == sourceLayers.size()) {
+        const auto ordered = selectedLayersOrdered();
+        for (Layer *layer : ordered)
+            if (layer->isTileLayer())
+                result.append(static_cast<TileLayer*>(layer));
+        return result;
+    }
+
+    LayerIterator targetIt(mMap.get(), Layer::TileLayerType);
+
+    auto findTargetLayer = [&] (const QString &name) -> TileLayer* {
+        Layer *startingLayer = targetIt.currentLayer();
+
+        if (startingLayer) {
+            // Search for the next matching layer, starting off where we
+            // stopped last time.
+            while (Layer *layer = targetIt.next())
+                if (layer->name() == name && !contains(result, layer))
+                    return static_cast<TileLayer*>(layer);
+        }
+
+        // Make sure we're at the front
+        targetIt.toFront();
+
+        // Search from the beginning if we previously didn't, in case
+        // the layer order does not match.
+        while (Layer *layer = targetIt.next()) {
+            if (layer == startingLayer) // stop if we reached start again
+                break;
+            if (layer->name() == name && !contains(result, layer))
+                return static_cast<TileLayer*>(layer);
+        }
+
+        return nullptr;
+    };
+
+    // For each source layer, find a target layer with matching name, while
+    // only using each target layer once.
+    for (const TileLayer *sourceLayer : sourceLayers)
+        result.append(findTargetLayer(sourceLayer->name()));
+
+    return result;
+}
+
 /**
  * Paints the tile layers present in the given \a map onto this map. Matches
  * layers by name and creates new layers when they could not be found.
  *
  * In case the \a map only contains a single tile layer, it is always painted
- * into the current tile layer. This happens also for unnamed layers. In these
- * cases, the layers are skipped when the current layer isn't a tile layer.
+ * into the current tile layer.
  *
- * If the matched target layer is locked it is also skipped.
+ * If the matched target layer is locked it is skipped.
  *
  * \a mergeable indicates whether the paint operations performed by this
  * function are mergeable with previous compatible paint operations.
@@ -875,73 +934,110 @@ SharedTileset MapDocument::replaceTileset(int index, const SharedTileset &tilese
  * In this case it is the responsibility of the caller to emit this signal for
  * each affected tile layer.
  */
-void MapDocument::paintTileLayers(const Map *map, bool mergeable,
+void MapDocument::paintTileLayers(const Map &map, bool mergeable,
                                   QVector<SharedTileset> *missingTilesets,
                                   QHash<TileLayer*, QRegion> *paintedRegions)
 {
-    TileLayer *currentTileLayer = mCurrentLayer ? mCurrentLayer->asTileLayer() : nullptr;
+    struct PaintPair {
+        const TileLayer *source;
+        TileLayer *target;
+    };
 
-    LayerIterator it(map, Layer::TileLayerType);
-    const bool isMultiLayer = it.next() && it.next();
+    QList<const TileLayer *> sourceLayers;
+    for (const Layer *layer : map.tileLayers())
+        sourceLayers.append(static_cast<const TileLayer*>(layer));
 
-    it.toFront();
-    while (auto tileLayer = static_cast<TileLayer*>(it.next())) {
-        TileLayer *targetLayer = currentTileLayer;
-        bool addLayer = false;
+    const QList<TileLayer *> targetLayers = findTargetLayers(sourceLayers);
+    QHash<TileLayer*, QRegion> localPaintedRegions;
+    auto &regions = paintedRegions ? *paintedRegions : localPaintedRegions;
 
-        // When the map contains only a single layer, always paint it into
-        // the current layer. This makes sure you can still take pieces from
-        // one layer and draw them into another.
-        if (isMultiLayer && !tileLayer->name().isEmpty()) {
-            targetLayer = static_cast<TileLayer*>(mMap->findLayer(tileLayer->name(), Layer::TileLayerType));
-            if (!targetLayer) {
-                // Create a layer with this name
-                targetLayer = new TileLayer(tileLayer->name(), 0, 0,
-                                            mMap->width(),
-                                            mMap->height());
-                addLayer = true;
-            }
+    TileLayer *lastTarget = nullptr;
+
+    for (int i = 0; i < sourceLayers.size(); ++i) {
+        auto source = sourceLayers[i];
+        auto target = targetLayers[i];
+
+        const QRegion editedRegion = source->region();
+        if (editedRegion.isEmpty())
+            continue;
+
+        std::unique_ptr<TileLayer> newLayer;
+
+        if (!target) {
+            // Create a layer with this name
+            newLayer = std::make_unique<TileLayer>(source->name(), 0, 0,
+                                                   mMap->width(),
+                                                   mMap->height());
+            newLayer->setOpacity(source->opacity());
+            newLayer->setTintColor(source->tintColor());
+            target = newLayer.get();
         }
 
-        if (!targetLayer)
+        if (!target->isUnlocked())
             continue;
-        if (!targetLayer->isUnlocked())
-            continue;
-        if (!mMap->infinite() && !targetLayer->rect().intersects(tileLayer->bounds()))
+        if (!mMap->infinite() && !target->rect().intersects(source->bounds()))
             continue;
 
-        PaintTileLayer *paint = new PaintTileLayer(this,
-                                                   targetLayer,
-                                                   tileLayer->x(),
-                                                   tileLayer->y(),
-                                                   tileLayer);
+        PaintTileLayer *paintCommand = new PaintTileLayer(this,
+                                                          target,
+                                                          source->x(),
+                                                          source->y(),
+                                                          source,
+                                                          editedRegion);
 
         if (missingTilesets && !missingTilesets->isEmpty()) {
             for (const SharedTileset &tileset : qAsConst(*missingTilesets)) {
                 if (!mMap->tilesets().contains(tileset))
-                    new AddTileset(this, tileset, paint);
+                    new AddTileset(this, tileset, paintCommand);
             }
 
             missingTilesets->clear();
         }
 
-        if (addLayer) {
-            new AddLayer(this,
-                         mMap->layerCount(), targetLayer, nullptr,
-                         paint);
+        if (newLayer) {
+            int index = mMap->layerCount();
+            GroupLayer *parent = nullptr;
+
+            if (lastTarget) {
+                // If we need to create a layer, insert it right after the
+                // previous layer we painted to.
+                index = lastTarget->siblingIndex() + 1;
+                parent = lastTarget->parentLayer();
+            } else {
+                // In case we haven't painted on another layer yet, try to
+                // insert it right before the next layer we'll paint to.
+                auto it = std::find_if(targetLayers.cbegin() + i + 1,
+                                       targetLayers.cend(),
+                                       [] (TileLayer *layer) { return layer != nullptr; });
+                if (it != targetLayers.cend()) {
+                    index = (*it)->siblingIndex();
+                    parent = (*it)->parentLayer();
+                }
+            }
+
+            new AddLayer(this, index, newLayer.release(), parent, paintCommand);
         }
 
-        paint->setMergeable(mergeable);
-        undoStack()->push(paint);
+        lastTarget = target;
 
-        const QRegion editedRegion = tileLayer->region();
-        if (paintedRegions)
-            (*paintedRegions)[targetLayer] |= editedRegion;
-        else
-            emit regionEdited(editedRegion, targetLayer);
+        paintCommand->setMergeable(mergeable);
+        undoStack()->push(paintCommand);
+
+        regions[target] |= editedRegion;
 
         mergeable = true; // further paints are always mergeable
     }
+
+    if (!paintedRegions) {
+        QHashIterator<TileLayer*, QRegion> ri(regions);
+        while (ri.hasNext()) {
+            TileLayer *layer = ri.next().key();
+            if (layer->map() != this->map())  // script might have removed the layer on regionEdited
+                continue;
+
+            emit regionEdited(ri.value(), layer);
+        }
+    }
 }
 
 void MapDocument::replaceObjectTemplate(const ObjectTemplate *oldObjectTemplate,
@@ -963,8 +1059,33 @@ void MapDocument::setSelectedArea(const QRegion &selection)
     }
 }
 
+static QList<Layer *> sortLayers(const Map &map, const QList<Layer *> &layers)
+{
+    if (layers.size() < 2)
+        return layers;
+
+    QList<Layer *> sorted;
+    sorted.reserve(layers.size());
+
+    LayerIterator iterator(&map);
+    while (Layer *layer = iterator.next()) {
+        if (layers.contains(layer))
+            sorted.append(layer);
+    }
+
+    return sorted;
+}
+
+QList<Layer *> MapDocument::selectedLayersOrdered() const
+{
+    return sortLayers(*mMap, mSelectedLayers);
+}
+
 static QList<MapObject *> sortObjects(const Map &map, const QList<MapObject *> &objects)
 {
+    if (objects.size() < 2)
+        return objects;
+
     QList<MapObject *> sorted;
     sorted.reserve(objects.size());
 
@@ -1083,13 +1204,13 @@ void MapDocument::setHoveredMapObject(MapObject *object)
  * in the current map document and all missing tilesets will be added to
  * the current map document.
  */
-void MapDocument::unifyTilesets(Map *map)
+void MapDocument::unifyTilesets(Map &map)
 {
     QList<QUndoCommand*> undoCommands;
     QVector<SharedTileset> availableTilesets = mMap->tilesets();
 
     // Iterate over a copy because map->replaceTileset may invalidate iterator
-    const QVector<SharedTileset> tilesets = map->tilesets();
+    const QVector<SharedTileset> tilesets = map.tilesets();
     for (const SharedTileset &tileset : tilesets) {
         if (availableTilesets.contains(tileset))
             continue;
@@ -1113,7 +1234,7 @@ void MapDocument::unifyTilesets(Map *map)
             }
         }
 
-        map->replaceTileset(tileset, replacement);
+        map.replaceTileset(tileset, replacement);
     }
 
     if (!undoCommands.isEmpty()) {
@@ -1130,7 +1251,7 @@ void MapDocument::unifyTilesets(Map *map)
  * and adds tilesets to \a missingTilesets whenever there is a tileset without
  * replacement in this map.
  */
-void MapDocument::unifyTilesets(Map *map, QVector<SharedTileset> &missingTilesets)
+void MapDocument::unifyTilesets(Map &map, QVector<SharedTileset> &missingTilesets)
 {
     QVector<SharedTileset> availableTilesets = mMap->tilesets();
     for (const SharedTileset &tileset : qAsConst(missingTilesets))
@@ -1138,7 +1259,7 @@ void MapDocument::unifyTilesets(Map *map, QVector<SharedTileset> &missingTileset
             availableTilesets.append(tileset);
 
     // Iterate over a copy because map->replaceTileset may invalidate iterator
-    const QVector<SharedTileset> tilesets = map->tilesets();
+    const QVector<SharedTileset> tilesets = map.tilesets();
     for (const SharedTileset &tileset : tilesets) {
         // tileset already added
         if (availableTilesets.contains(tileset))
@@ -1154,7 +1275,7 @@ void MapDocument::unifyTilesets(Map *map, QVector<SharedTileset> &missingTileset
         }
 
         // replacement tileset found, change given map
-        map->replaceTileset(tileset, replacement);
+        map.replaceTileset(tileset, replacement);
     }
 }
 
@@ -1445,8 +1566,8 @@ void MapDocument::moveObjectsToGroup(const QList<MapObject *> &objects,
     undoStack()->endMacro();
 }
 
-typedef QHash<ObjectGroup*, RangeSet<int>>           Ranges;
-typedef QHashIterator<ObjectGroup*, RangeSet<int>>   RangesIterator;
+using Ranges = QHash<ObjectGroup *, RangeSet<int>>;
+using RangesIterator = QHashIterator<ObjectGroup *, RangeSet<int>>;
 
 static Ranges computeRanges(const QList<MapObject *> &objects)
 {
