@@ -34,7 +34,9 @@
 #include "createtextobjecttool.h"
 #include "createtileobjecttool.h"
 #include "documentmanager.h"
+#include "editablemanager.h"
 #include "editablemap.h"
+#include "editablewangset.h"
 #include "editpolygontool.h"
 #include "eraser.h"
 #include "filechangedwarning.h"
@@ -62,9 +64,6 @@
 #include "shapefilltool.h"
 #include "stampbrush.h"
 #include "templatesdock.h"
-#include "terrain.h"
-#include "terrainbrush.h"
-#include "terraindock.h"
 #include "tile.h"
 #include "tileselectiontool.h"
 #include "tilesetdock.h"
@@ -153,7 +152,6 @@ MapEditor::MapEditor(QObject *parent)
     , mObjectsDock(new ObjectsDock(mMainWindow))
     , mTemplatesDock(new TemplatesDock(mMainWindow))
     , mTilesetDock(new TilesetDock(mMainWindow))
-    , mTerrainDock(new TerrainDock(mMainWindow))
     , mWangDock(new WangDock(mMainWindow))
     , mMiniMapDock(new MiniMapDock(mMainWindow))
     , mLayerComboBox(new TreeViewComboBox)
@@ -179,7 +177,6 @@ MapEditor::MapEditor(QObject *parent)
     mToolSpecificToolBar->setObjectName(QLatin1String("toolSpecificToolBar"));
 
     mStampBrush = new StampBrush(this);
-    mTerrainBrush = new TerrainBrush(this);
     mWangBrush = new WangBrush(this);
     mBucketFillTool = new BucketFillTool(this);
     mEditPolygonTool = new EditPolygonTool(this);
@@ -193,7 +190,6 @@ MapEditor::MapEditor(QObject *parent)
     CreateObjectTool *textObjectsTool = new CreateTextObjectTool(this);
 
     mToolsToolBar->addAction(mToolManager->registerTool(mStampBrush));
-    mToolsToolBar->addAction(mToolManager->registerTool(mTerrainBrush));
     mToolsToolBar->addAction(mToolManager->registerTool(mWangBrush));
     mToolsToolBar->addAction(mToolManager->registerTool(mBucketFillTool));
     mToolsToolBar->addAction(mToolManager->registerTool(mShapeFillTool));
@@ -275,13 +271,6 @@ MapEditor::MapEditor(QObject *parent)
     connect(mBucketFillTool, &BucketFillTool::wangFillChanged, this, &MapEditor::setWangFill);
     connect(mShapeFillTool, &ShapeFillTool::wangFillChanged, this, &MapEditor::setWangFill);
 
-    connect(mTerrainDock, &TerrainDock::currentTerrainChanged,
-            mTerrainBrush, &TerrainBrush::setTerrain);
-    connect(mTerrainDock, &TerrainDock::selectTerrainBrush,
-            this, &MapEditor::selectTerrainBrush);
-    connect(mTerrainBrush, &TerrainBrush::terrainCaptured,
-            mTerrainDock, &TerrainDock::setCurrentTerrain);
-
     connect(mWangDock, &WangDock::currentWangSetChanged,
             mBucketFillTool, &BucketFillTool::setWangSet);
     connect(mWangDock, &WangDock::currentWangSetChanged,
@@ -290,10 +279,14 @@ MapEditor::MapEditor(QObject *parent)
             mStampBrush, &StampBrush::setWangSet);
     connect(mWangDock, &WangDock::currentWangSetChanged,
             mWangBrush, &WangBrush::wangSetChanged);
+    connect(mWangDock, &WangDock::currentWangSetChanged,
+            this, &MapEditor::currentWangSetChanged);
+    connect(mWangDock, &WangDock::wangColorChanged,
+            this, &MapEditor::currentWangColorIndexChanged);
     connect(mWangDock, &WangDock::selectWangBrush,
             this, &MapEditor::selectWangBrush);
     connect(mWangDock, &WangDock::wangColorChanged,
-            mWangBrush, &WangBrush::wangColorChanged);
+            mWangBrush, &WangBrush::setColor);
     connect(mWangBrush, &WangBrush::colorCaptured,
             mWangDock, &WangDock::onColorCaptured);
 
@@ -311,6 +304,8 @@ MapEditor::MapEditor(QObject *parent)
     connect(prefs, &Preferences::languageChanged, this, &MapEditor::retranslateUi);
     connect(prefs, &Preferences::showTileCollisionShapesChanged,
             this, &MapEditor::showTileCollisionShapesChanged);
+    connect(prefs, &Preferences::parallaxEnabledChanged,
+            this, &MapEditor::parallaxEnabledChanged);
     connect(prefs, &Preferences::aboutToSwitchSession,
             this, [this] { if (mCurrentMapDocument) saveDocumentState(mCurrentMapDocument); });
 
@@ -347,6 +342,7 @@ void MapEditor::addDocument(Document *document)
 
     auto prefs = Preferences::instance();
     scene->setShowTileCollisionShapes(prefs->showTileCollisionShapes());
+    scene->setParallaxEnabled(prefs->parallaxEnabled());
     scene->setMapDocument(mapDocument);
     view->setScene(scene);
 
@@ -362,6 +358,9 @@ void MapEditor::removeDocument(Document *document)
     Q_ASSERT(mapDocument);
     Q_ASSERT(mWidgetForMap.contains(mapDocument));
 
+    if (mapDocument == mCurrentMapDocument)
+        setCurrentDocument(nullptr);
+
     MapView *mapView = mWidgetForMap.take(mapDocument);
     // remove first, to keep it valid while the current widget changes
     mWidgetStack->removeWidget(mapView);
@@ -400,7 +399,6 @@ void MapEditor::setCurrentDocument(Document *document)
     mUndoDock->setStack(document ? document->undoStack() : nullptr);
     mObjectsDock->setMapDocument(mapDocument);
     mTilesetDock->setMapDocument(mapDocument);
-    mTerrainDock->setDocument(mapDocument);
     mWangDock->setDocument(mapDocument);
     mMiniMapDock->setMapDocument(mapDocument);
 
@@ -417,10 +415,7 @@ void MapEditor::setCurrentDocument(Document *document)
             mZoomable->setComboBox(mZoomComboBox.get());
         }
 
-        connect(mCurrentMapDocument, &MapDocument::currentObjectChanged,
-                this, [this, mapDocument] { mPropertiesDock->setDocument(mapDocument); });
-
-        connect(mapView, &MapView::focused,
+        connect(mCurrentMapDocument, &MapDocument::currentObjectSet,
                 this, [this, mapDocument] { mPropertiesDock->setDocument(mapDocument); });
 
         mReversingProxyModel->setSourceModel(mapDocument->layerModel());
@@ -483,7 +478,6 @@ QList<QDockWidget *> MapEditor::dockWidgets() const
         mObjectsDock,
         mTemplatesDock,
         mTilesetDock,
-        mTerrainDock,
         mWangDock,
         mMiniMapDock,
         mTileStampsDock
@@ -583,17 +577,14 @@ void MapEditor::resetLayout()
     mMainWindow->tabifyDockWidget(mMiniMapDock, mObjectsDock);
     mMainWindow->tabifyDockWidget(mObjectsDock, mLayerDock);
 
-    mMainWindow->addDockWidget(Qt::RightDockWidgetArea, mTerrainDock);
     mMainWindow->addDockWidget(Qt::RightDockWidgetArea, mWangDock);
     mMainWindow->addDockWidget(Qt::RightDockWidgetArea, mTilesetDock);
-    mMainWindow->tabifyDockWidget(mTerrainDock, mWangDock);
     mMainWindow->tabifyDockWidget(mWangDock, mTilesetDock);
 
     // These dock widgets may not be immediately useful to many people, so
     // they are hidden by default.
     mUndoDock->setVisible(false);
     mTemplatesDock->setVisible(false);
-    mWangDock->setVisible(false);
     mTileStampsDock->setVisible(false);
 }
 
@@ -698,25 +689,6 @@ void MapEditor::updateActiveUndoStack()
     }
 }
 
-static void normalizeTileLayerPositionsAndMapSize(Map *map)
-{
-    LayerIterator it(map, Layer::TileLayerType);
-
-    QRect contentRect;
-    while (auto tileLayer = static_cast<TileLayer*>(it.next()))
-        contentRect |= tileLayer->region().boundingRect();
-
-    if (!contentRect.isEmpty()) {
-        QPoint offset = contentRect.topLeft();
-        it.toFront();
-        while (auto tileLayer = static_cast<TileLayer*>(it.next()))
-            tileLayer->setPosition(tileLayer->position() - offset);
-
-        map->setWidth(contentRect.width());
-        map->setHeight(contentRect.height());
-    }
-}
-
 void MapEditor::paste(ClipboardManager::PasteFlags flags)
 {
     if (!mCurrentMapDocument)
@@ -727,39 +699,37 @@ void MapEditor::paste(ClipboardManager::PasteFlags flags)
     if (!map)
         return;
 
-    Map *mapPtr = map.get();
-
     bool tilesetsUnified = false;
 
     if (flags & ClipboardManager::PasteInPlace)
         mCurrentMapDocument->undoStack()->beginMacro(tr("Paste in Place"));
 
-    LayerIterator tileLayerIterator(mapPtr, Layer::TileLayerType);
-    if (tileLayerIterator.next()) {
-        if (flags & ClipboardManager::PasteInPlace) {
-            QVector<SharedTileset> missingTilesets;
-            mCurrentMapDocument->unifyTilesets(mapPtr, missingTilesets);
-            mCurrentMapDocument->paintTileLayers(mapPtr, false, &missingTilesets);
-            tilesetsUnified = missingTilesets.isEmpty();
-        } else {
-            // Reset selection and paste into the stamp brush
-            MapDocumentActionHandler::instance()->selectNone();
-            normalizeTileLayerPositionsAndMapSize(mapPtr);
-            setStamp(TileStamp(std::move(map))); // TileStamp takes ownership
-            mToolManager->selectTool(mStampBrush);
-        }
+    const bool hasTileLayers = LayerIterator(map.get(), Layer::TileLayerType).next();
+    if (hasTileLayers && (flags & ClipboardManager::PasteInPlace)) {
+        QVector<SharedTileset> missingTilesets;
+        mCurrentMapDocument->unifyTilesets(*map, missingTilesets);
+        mCurrentMapDocument->paintTileLayers(*map, false, &missingTilesets);
+        tilesetsUnified = missingTilesets.isEmpty();
     }
 
-    LayerIterator objectGroupIterator(mapPtr, Layer::ObjectGroupType);
+    LayerIterator objectGroupIterator(map.get(), Layer::ObjectGroupType);
     if (ObjectGroup *objectGroup = static_cast<ObjectGroup*>(objectGroupIterator.next())) {
         if (!tilesetsUnified)
-            mCurrentMapDocument->unifyTilesets(mapPtr);
+            mCurrentMapDocument->unifyTilesets(*map);
 
         // todo: Handle multiple object groups
         const MapView *view = currentMapView();
         clipboardManager->pasteObjectGroup(objectGroup, mCurrentMapDocument, view, flags);
     }
 
+    if (hasTileLayers && !(flags & ClipboardManager::PasteInPlace)) {
+        // Reset selection and paste into the stamp brush
+        MapDocumentActionHandler::instance()->selectNone();
+        map->normalizeTileLayerPositionsAndMapSize();
+        setStamp(TileStamp(std::move(map)));
+        mToolManager->selectTool(mStampBrush);
+    }
+
     if (flags & ClipboardManager::PasteInPlace)
         mCurrentMapDocument->undoStack()->endMacro();
 }
@@ -801,28 +771,26 @@ void MapEditor::setStamp(const TileStamp &stamp)
 
     // When selecting a new stamp, it makes sense to switch to a stamp tool
     AbstractTool *selectedTool = mToolManager->selectedTool();
-    if (selectedTool != mStampBrush
-            && selectedTool != mBucketFillTool
-            && selectedTool != mShapeFillTool)
+    if (!selectedTool || !selectedTool->usesSelectedTiles())
         mToolManager->selectTool(mStampBrush);
 
     mTilesetDock->selectTilesInStamp(stamp);
 }
 
-void MapEditor::selectTerrainBrush()
-{
-    mToolManager->selectTool(mTerrainBrush);
-}
-
 void MapEditor::selectWangBrush()
 {
+    // Don't switch tools when the current tool also uses Wang sets
+    AbstractTool *selectedTool = mToolManager->selectedTool();
+    if (selectedTool && selectedTool->usesWangSets())
+        return;
+
     mToolManager->selectTool(mWangBrush);
 }
 
 void MapEditor::currentWidgetChanged()
 {
-    auto mapView = static_cast<MapView*>(mWidgetStack->currentWidget());
-    setCurrentDocument(mapView ? mapView->mapScene()->mapDocument() : nullptr);
+    if (!mWidgetStack->currentWidget())
+        setCurrentDocument(nullptr);
 }
 
 void MapEditor::cursorChanged(const QCursor &cursor)
@@ -910,7 +878,7 @@ void MapEditor::handleExternalTilesetsAndImages(const QStringList &fileNames,
             tileset = tilesetFormat->read(fileName);
             if (tileset) {
                 tileset->setFileName(fileName);
-                tileset->setFormat(tilesetFormat);
+                tileset->setFormat(tilesetFormat->shortName());
                 tilesets.append(tileset);
                 continue;
             } else {
@@ -960,6 +928,8 @@ void MapEditor::handleExternalTilesetsAndImages(const QStringList &fileNames,
         for (; it != end; ++it)
             undoStack->push(new AddTileset(mCurrentMapDocument, *it));
         undoStack->endMacro();
+
+        mTilesetDock->setCurrentTileset(tilesets.last());
     }
 }
 
@@ -1002,7 +972,7 @@ void MapEditor::setupQuickStamps()
         connect(createStamp, &QShortcut::activated, [=] { mTileStampManager->createQuickStamp(i); });
 
         // Set up shortcut for extending this quick stamp
-        QShortcut *extendStamp = new QShortcut(Qt::CTRL + Qt::SHIFT + key, mMainWindow);
+        QShortcut *extendStamp = new QShortcut((Qt::CTRL | Qt::SHIFT) + key, mMainWindow);
         connect(extendStamp, &QShortcut::activated, [=] { mTileStampManager->extendQuickStamp(i); });
     }
 
@@ -1018,10 +988,16 @@ void MapEditor::retranslateUi()
 
 void MapEditor::showTileCollisionShapesChanged(bool enabled)
 {
-    for (auto mapView : qAsConst(mWidgetForMap))
+    for (MapView *mapView : qAsConst(mWidgetForMap))
         mapView->mapScene()->setShowTileCollisionShapes(enabled);
 }
 
+void MapEditor::parallaxEnabledChanged(bool enabled)
+{
+    for (MapView *mapView : qAsConst(mWidgetForMap))
+        mapView->mapScene()->setParallaxEnabled(enabled);
+}
+
 void MapEditor::setCurrentTileset(const SharedTileset &tileset)
 {
     mTilesetDock->setCurrentTileset(tileset);
@@ -1054,8 +1030,25 @@ void MapEditor::setCurrentBrush(EditableMap *editableMap)
     setStamp(TileStamp(editableMap->map()->clone()));
 }
 
-AbstractTool *MapEditor::selectedTool() const {
+EditableWangSet *MapEditor::currentWangSet() const
+{
+    auto currentWangSet = mWangDock->currentWangSet();
+    if (!currentWangSet)
+        return nullptr;
+
+    return EditableManager::instance().editableWangSet(currentWangSet);
+}
+
+int MapEditor::currentWangColorIndex() const
+{
+    return mWangDock->currentWangColor();
+}
+
+AbstractTool *MapEditor::selectedTool() const
+{
     return mSelectedTool;
 }
 
 } // namespace Tiled
+
+#include "moc_mapeditor.cpp"
