@@ -158,8 +158,8 @@ MapItem::MapItem(const MapDocumentPtr &mapDocument, DisplayMode displayMode,
     connect(mapDocument.data(), &MapDocument::regionChanged, this, &MapItem::repaintRegion);
     connect(mapDocument.data(), &MapDocument::tileLayerChanged, this, &MapItem::tileLayerChanged);
     connect(mapDocument.data(), &MapDocument::layerAdded, this, &MapItem::layerAdded);
+    connect(mapDocument.data(), &MapDocument::layerAboutToBeRemoved, this, &MapItem::layerAboutToBeRemoved);
     connect(mapDocument.data(), &MapDocument::layerRemoved, this, &MapItem::layerRemoved);
-    connect(mapDocument.data(), &MapDocument::imageLayerChanged, this, &MapItem::imageLayerChanged);
     connect(mapDocument.data(), &MapDocument::selectedLayersChanged, this, &MapItem::updateSelectedLayersHighlight);
     connect(mapDocument.data(), &MapDocument::tilesetTilePositioningChanged, this, &MapItem::adaptToTilesetTileSizeChanges);
     connect(mapDocument.data(), &MapDocument::tileImageSourceChanged, this, &MapItem::adaptToTileSizeChanges);
@@ -266,7 +266,7 @@ void MapItem::updateLayerPositions()
 
     for (LayerItem *layerItem : qAsConst(mLayerItems)) {
         const Layer &layer = *layerItem->layer();
-        layerItem->setPos(layer.offset() + mapScene->parallaxOffset(layer));
+        layerItem->setPos(mapScene->layerItemPosition(layer));
     }
 
     if (mDisplayMode == Editable) {
@@ -371,6 +371,9 @@ void MapItem::documentChanged(const ChangeEvent &change)
     case ChangeEvent::LayerChanged:
         layerChanged(static_cast<const LayerChangeEvent&>(change));
         break;
+    case ChangeEvent::ImageLayerChanged:
+        imageLayerChanged(static_cast<const ImageLayerChangeEvent&>(change).imageLayer());
+        break;
     case ChangeEvent::MapObjectAboutToBeRemoved: {
         auto &e = static_cast<const MapObjectEvent&>(change);
         deleteObjectItem(e.objectGroup->objectAt(e.index));
@@ -435,12 +438,27 @@ void MapItem::layerAdded(Layer *layer)
         mLayerItems.value(sibling)->setZValue(z++);
 
     updateBoundingRect();
+    updateSelectedLayersHighlight();
+}
+
+void MapItem::layerAboutToBeRemoved(GroupLayer *parentLayer, int index)
+{
+    // Fix up the Z value of the items of the siblings, before the layer is removed
+    const auto siblings = parentLayer ? parentLayer->layers()
+                                      : mMapDocument->map()->layers();
+    const Layer *layer = siblings.at(index);
+
+    int z = 0;
+    for (auto sibling : siblings)
+        if (sibling != layer)
+            mLayerItems.value(sibling)->setZValue(z++);
 }
 
 void MapItem::layerRemoved(Layer *layer)
 {
     deleteLayerItems(layer);
     updateBoundingRect();
+    updateSelectedLayersHighlight();
 }
 
 /**
@@ -480,10 +498,8 @@ void MapItem::layerChanged(const LayerChangeEvent &change)
             multiplier = opacityFactor;
     }
 
-    const QPointF parallaxOffset = static_cast<MapScene*>(scene())->parallaxOffset(*layer);
-
     layerItem->setOpacity(layer->opacity() * multiplier);
-    layerItem->setPos(layer->offset() + parallaxOffset);
+    layerItem->setPos(static_cast<MapScene*>(scene())->layerItemPosition(*layer));
 
     updateBoundingRect();   // possible layer offset change
 }
@@ -720,7 +736,7 @@ LayerItem *MapItem::createLayerItem(Layer *layer)
     // MapItem constructor and the layer will be positioned by a call to
     // updateLayerPositions from the MapScene.
     if (const MapScene *mapScene = static_cast<MapScene*>(scene()))
-        layerItem->setPos(layer->offset() + mapScene->parallaxOffset(*layer));
+        layerItem->setPos(mapScene->layerItemPosition(*layer));
 
     layerItem->setVisible(layer->isVisible());
     layerItem->setEnabled(mDisplayMode == Editable);
