@@ -52,11 +52,11 @@ const ObjectIconManager &ObjectIconManager::instance()
     return objectIconManager;
 }
 
-const QIcon &ObjectIconManager::iconForObject(MapObject *object) const
+const QIcon &ObjectIconManager::iconForObject(const MapObject &object) const
 {
-    switch (object->shape()) {
+    switch (object.shape()) {
     case MapObject::Rectangle:
-        return object->isTileObject() ? mImageIcon : mRectangleIcon;
+        return object.isTileObject() ? mImageIcon : mRectangleIcon;
     case MapObject::Polygon:
         return mPolygonIcon;
     case MapObject::Polyline:
@@ -170,7 +170,7 @@ QVariant MapObjectModel::data(const QModelIndex &index, int role) const
             break;
         case Qt::DecorationRole:
             if (index.column() == Name)
-                return ObjectIconManager::instance().iconForObject(mapObject);
+                return ObjectIconManager::instance().iconForObject(*mapObject);
             break;
         case Qt::ForegroundRole:
             if (index.column() == 1) {
@@ -261,7 +261,7 @@ bool MapObjectModel::setData(const QModelIndex &index, const QVariant &value,
             const bool visible = (c == Qt::Checked);
             if (visible != layer->isVisible()) {
                 QUndoCommand *command = new SetLayerVisible(mMapDocument,
-                                                            layer,
+                                                            { layer },
                                                             visible);
                 mMapDocument->undoStack()->push(command);
             }
@@ -270,7 +270,8 @@ bool MapObjectModel::setData(const QModelIndex &index, const QVariant &value,
         case Qt::EditRole: {
             const QString newName = value.toString();
             if (layer->name() != newName) {
-                SetLayerName *rename = new SetLayerName(mMapDocument, layer,
+                SetLayerName *rename = new SetLayerName(mMapDocument,
+                                                        { layer },
                                                         newName);
                 mMapDocument->undoStack()->push(rename);
             }
@@ -523,8 +524,6 @@ void MapObjectModel::documentChanged(const ChangeEvent &change)
 
         break;
     }
-    case ChangeEvent::TileLayerChanged:
-        break;
     case ChangeEvent::MapObjectsAboutToBeRemoved:
     case ChangeEvent::MapObjectsAdded:
     case ChangeEvent::MapObjectsRemoved:
@@ -561,7 +560,7 @@ void MapObjectModel::documentChanged(const ChangeEvent &change)
 
         break;
     }
-    case ChangeEvent::ObjectGroupChanged:
+    default:
         break;
     }
 }
@@ -580,3 +579,5 @@ void MapObjectModel::emitDataChanged(const QList<MapObject *> &objects,
                          roles);
     }
 }
+
+#include "moc_mapobjectmodel.cpp"
