@@ -26,6 +26,7 @@
 #include "abstracttool.h"
 #include "addremovemapobject.h"
 #include "containerhelpers.h"
+#include "debugdrawitem.h"
 #include "documentmanager.h"
 #include "map.h"
 #include "mapobject.h"
@@ -50,8 +51,11 @@
 
 using namespace Tiled;
 
+SessionOption<bool> MapScene::enableWorlds { "mapScene.enableWorlds", true };
+
 MapScene::MapScene(QObject *parent)
     : QGraphicsScene(parent)
+    , mWorldsEnabled(enableWorlds)
 {
     updateDefaultBackgroundColor();
 
@@ -70,10 +74,19 @@ MapScene::MapScene(QObject *parent)
     // Install an event filter so that we can get key events on behalf of the
     // active tool without having to have the current focus.
     qApp->installEventFilter(this);
+
+    mEnableWorldsCallback = enableWorlds.onChange([this] { setWorldsEnabled(enableWorlds); });
+
+#ifdef QT_DEBUG
+    mDebugDrawItem = new DebugDrawItem;
+    addItem(mDebugDrawItem);
+#endif
 }
 
 MapScene::~MapScene()
 {
+    enableWorlds.unregister(mEnableWorldsCallback);
+
     qApp->removeEventFilter(this);
 }
 
@@ -88,6 +101,8 @@ void MapScene::setMapDocument(MapDocument *mapDocument)
     mMapDocument = mapDocument;
 
     if (mMapDocument) {
+        connect(mMapDocument, &MapDocument::changed,
+                this, &MapScene::changeEvent);
         connect(mMapDocument, &MapDocument::mapChanged,
                 this, &MapScene::mapChanged);
         connect(mMapDocument, &MapDocument::tilesetTilePositioningChanged,
@@ -112,6 +127,15 @@ void MapScene::setShowTileCollisionShapes(bool enabled)
         mapItem->setShowTileCollisionShapes(enabled);
 }
 
+void MapScene::setParallaxEnabled(bool enabled)
+{
+    if (mParallaxEnabled == enabled)
+        return;
+
+    mParallaxEnabled = enabled;
+    emit parallaxParametersChanged();
+}
+
 /**
  * Returns the bounding rect of the map. This can be different from the
  * sceneRect() when multiple maps are displayed.
@@ -152,6 +176,69 @@ void MapScene::setSelectedTool(AbstractTool *tool)
     }
 }
 
+/**
+ * Sets the area of the scene that is currently visible in the MapView.
+ */
+void MapScene::setViewRect(const QRectF &rect)
+{
+    if (mViewRect == rect)
+        return;
+
+    mViewRect = rect;
+
+    if (mParallaxEnabled)
+        emit parallaxParametersChanged();
+}
+
+void MapScene::setOverrideBackgroundColor(QColor backgroundColor)
+{
+    if (mOverrideBackgroundColor == backgroundColor)
+        return;
+
+    mOverrideBackgroundColor = backgroundColor;
+    updateBackgroundColor();
+};
+
+/**
+ * Returns the position the given layer is supposed to have, taking into
+ * account its offset and the parallax factor along with the current view rect.
+ */
+QPointF MapScene::absolutePositionForLayer(const Layer &layer) const
+{
+    return layer.totalOffset() + parallaxOffset(layer);
+}
+
+/**
+ * Returns the position for a layer item.
+ *
+ * Since layer items are in a hierarchy where translation of the parent layer
+ * item affects the child layer items, the local offset is used instead of the
+ * total offset.
+ *
+ * Similarly, since the effective parallax factor is applied, this factor is
+ * ignored for group layers.
+ */
+QPointF MapScene::layerItemPosition(const Layer &layer) const
+{
+    return layer.offset() + (layer.isGroupLayer() ? QPointF()
+                                                  : parallaxOffset(layer));
+}
+
+/**
+ * Returns the parallax offset of the given layer, taking into account its
+ * parallax factor in combination with the current view rect.
+ */
+QPointF MapScene::parallaxOffset(const Layer &layer) const
+{
+    if (!mParallaxEnabled)
+        return {};
+
+    const QPointF parallaxFactor = layer.effectiveParallaxFactor();
+    const QPointF viewCenter = mViewRect.center() - mapDocument()->map()->parallaxOrigin();
+    return QPointF((1.0 - parallaxFactor.x()) * viewCenter.x(),
+                   (1.0 - parallaxFactor.y()) * viewCenter.y());
+}
+
 /**
  * Refreshes the map scene.
  */
@@ -190,6 +277,7 @@ void MapScene::refreshScene()
 
                 auto mapItem = takeOrCreateMapItem(mapDocument, displayMode);
                 mapItem->setPos(mapEntry.rect.topLeft() - currentMapPosition);
+                mapItem->setVisible(mWorldsEnabled || mapDocument == mMapDocument);
                 mapItems.insert(mapDocument.data(), mapItem);
             }
         }
@@ -201,24 +289,37 @@ void MapScene::refreshScene()
     mMapItems.swap(mapItems);
     qDeleteAll(mapItems);       // delete all map items that didn't get reused
 
+    updateBackgroundColor();
     updateSceneRect();
 
-    const Map *map = mMapDocument->map();
-
-    if (map->backgroundColor().isValid())
-        setBackgroundBrush(map->backgroundColor());
-    else
-        setBackgroundBrush(mDefaultBackgroundColor);
-
     emit sceneRefreshed();
 }
 
 void MapScene::updateDefaultBackgroundColor()
 {
-    mDefaultBackgroundColor = QGuiApplication::palette().dark().color();
+    const QColor darkColor = QGuiApplication::palette().dark().color();
+    if (mDefaultBackgroundColor != darkColor) {
+        mDefaultBackgroundColor = darkColor;
+        updateBackgroundColor();
+    }
+}
 
-    if (!mMapDocument || !mMapDocument->map()->backgroundColor().isValid())
-        setBackgroundBrush(mDefaultBackgroundColor);
+void MapScene::updateBackgroundColor()
+{
+    if (mOverrideBackgroundColor.isValid()) {
+        setBackgroundBrush(mOverrideBackgroundColor);
+        return;
+    }
+
+    if (mMapDocument) {
+        const QColor &backgroundColor = mMapDocument->map()->backgroundColor();
+        if (backgroundColor.isValid()) {
+            setBackgroundBrush(backgroundColor);
+            return;
+        }
+    }
+
+    setBackgroundBrush(mDefaultBackgroundColor);
 }
 
 void MapScene::updateSceneRect()
@@ -231,6 +332,17 @@ void MapScene::updateSceneRect()
     setSceneRect(sceneRect);
 }
 
+void MapScene::setWorldsEnabled(bool enabled)
+{
+    if (mWorldsEnabled == enabled)
+        return;
+
+    mWorldsEnabled = enabled;
+
+    for (MapItem *mapItem : qAsConst(mMapItems))
+        mapItem->setVisible(mWorldsEnabled || mapItem->mapDocument() == mMapDocument);
+}
+
 MapItem *MapScene::takeOrCreateMapItem(const MapDocumentPtr &mapDocument, MapItem::DisplayMode displayMode)
 {
     // Try to reuse an existing map item
@@ -239,23 +351,33 @@ MapItem *MapScene::takeOrCreateMapItem(const MapDocumentPtr &mapDocument, MapIte
         mapItem = new MapItem(mapDocument, displayMode);
         mapItem->setShowTileCollisionShapes(mShowTileCollisionShapes);
         connect(mapItem, &MapItem::boundingRectChanged, this, &MapScene::updateSceneRect);
+        connect(this, &MapScene::parallaxParametersChanged, mapItem, &MapItem::updateLayerPositions);
         addItem(mapItem);
+        mapItem->updateLayerPositions();
     } else {
         mapItem->setDisplayMode(displayMode);
     }
     return mapItem;
 }
 
+void MapScene::changeEvent(const ChangeEvent &change)
+{
+    switch (change.type) {
+    case ChangeEvent::MapChanged:
+        if (static_cast<const MapChangeEvent&>(change).property == Map::ParallaxOriginProperty)
+            emit parallaxParametersChanged();
+        break;
+    default:
+        break;
+    }
+}
+
 /**
  * Updates the possibly changed background color.
  */
 void MapScene::mapChanged()
 {
-    const Map *map = mMapDocument->map();
-    if (map->backgroundColor().isValid())
-        setBackgroundBrush(map->backgroundColor());
-    else
-        setBackgroundBrush(mDefaultBackgroundColor);
+    updateBackgroundColor();
 }
 
 void MapScene::repaintTileset(Tileset *tileset)
@@ -287,7 +409,11 @@ bool MapScene::event(QEvent *event)
         if (mSelectedTool)
             mSelectedTool->mouseEntered();
         break;
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     case QEvent::Leave:
+#else
+    case QEvent::GraphicsSceneLeave:
+#endif
         mUnderMouse = false;
         if (mSelectedTool)
             mSelectedTool->mouseLeft();
@@ -325,8 +451,14 @@ void MapScene::mouseMoveEvent(QGraphicsSceneMouseEvent *mouseEvent)
 //        return;
 
     if (mSelectedTool) {
-        mSelectedTool->mouseMoved(mouseEvent->scenePos(),
-                                mouseEvent->modifiers());
+        const Qt::KeyboardModifiers newModifiers = mouseEvent->modifiers();
+
+        if (newModifiers != mCurrentModifiers) {
+            mSelectedTool->modifiersChanged(newModifiers);
+            mCurrentModifiers = newModifiers;
+        }
+
+        mSelectedTool->mouseMoved(mouseEvent->scenePos(), mCurrentModifiers);
         mouseEvent->accept();
     }
 }
@@ -452,7 +584,7 @@ bool MapScene::eventFilter(QObject *, QEvent *event)
     case QEvent::KeyPress:
     case QEvent::KeyRelease: {
             QKeyEvent *keyEvent = static_cast<QKeyEvent*>(event);
-            Qt::KeyboardModifiers newModifiers = keyEvent->modifiers();
+            const Qt::KeyboardModifiers newModifiers = keyEvent->modifiers();
 
             if (mSelectedTool && newModifiers != mCurrentModifiers) {
                 mSelectedTool->modifiersChanged(newModifiers);
@@ -466,3 +598,5 @@ bool MapScene::eventFilter(QObject *, QEvent *event)
 
     return false;
 }
+
+#include "moc_mapscene.cpp"
