@@ -40,9 +40,16 @@
 #include <QWheelEvent>
 
 #ifndef QT_NO_OPENGL
-#include <QOpenGLWidget>
+
+// Needed to avoid include issue when compiling with mingw_900
+#if defined(Q_OS_WIN) && QT_VERSION >= QT_VERSION_CHECK(6, 2, 0)
+#define WIN32_LEAN_AND_MEAN
+#include <windows.h>
 #endif
 
+#include <QOpenGLWidget>
+#endif // QT_NO_OPENGL
+
 using namespace Tiled;
 
 Preference<bool> MapView::ourAutoScrollingEnabled { "Interface/AutoScrolling", false };
