@@ -29,6 +29,7 @@
 #include "session.h"
 #include "tilesetmanager.h"
 
+#include <QApplication>
 #include <QDir>
 #include <QFileInfo>
 #include <QStandardPaths>
@@ -41,8 +42,13 @@ QString Preferences::mStartupProject;
 
 Preferences *Preferences::instance()
 {
-    if (!mInstance)
-        mInstance = new Preferences;
+    if (!mInstance) {
+        const auto iniFile = QDir(QApplication::applicationDirPath()).filePath(QStringLiteral("tiled.ini"));
+        if (QFileInfo::exists(iniFile) && QFileInfo(iniFile).isFile())
+            mInstance = new Preferences(iniFile);
+        else
+            mInstance = new Preferences;
+    }
     return mInstance;
 }
 
@@ -53,6 +59,27 @@ void Preferences::deleteInstance()
 }
 
 Preferences::Preferences()
+    : QSettings()
+{
+    initialize();
+}
+
+/**
+ * Uses the given settings file in INI format. This constructor is used for
+ * portable installs.
+ */
+Preferences::Preferences(const QString &fileName)
+    : QSettings(fileName, QSettings::IniFormat)
+    , mPortable(true)
+{
+    initialize();
+}
+
+Preferences::~Preferences()
+{
+}
+
+void Preferences::initialize()
 {
     // Make sure the data directory exists
     const QDir dataDir { dataLocation() };
@@ -103,10 +130,13 @@ Preferences::Preferences()
         setValue(QLatin1String("Install/DonationDialogTime"), donationDialogTime.toString(Qt::ISODate));
     }
     setValue(QLatin1String("Install/RunCount"), runCount() + 1);
-}
 
-Preferences::~Preferences()
-{
+    const auto oldGridMajorKey = QStringLiteral("Interface/GridMajor");
+    if (contains(oldGridMajorKey)) {
+        const int gridMajor = value(oldGridMajorKey).toInt();
+        setGridMajor(QSize(gridMajor, gridMajor));
+        remove(oldGridMajorKey);
+    }
 }
 
 bool Preferences::showGrid() const
@@ -134,6 +164,11 @@ bool Preferences::showObjectReferences() const
     return get("Interface/ShowObjectReferences", true);
 }
 
+bool Preferences::parallaxEnabled() const
+{
+    return get("Interface/ParallaxEnabled", true);
+}
+
 bool Preferences::snapToGrid() const
 {
     return get("Interface/SnapToGrid", false);
@@ -154,11 +189,21 @@ QColor Preferences::gridColor() const
     return get<QColor>("Interface/GridColor", Qt::black);
 }
 
+QColor Preferences::backgroundFadeColor() const
+{
+    return get<QColor>("Interface/BackgroundFadeColor", Qt::black);
+}
+
 int Preferences::gridFine() const
 {
     return get<int>("Interface/GridFine", 4);
 }
 
+QSize Preferences::gridMajor() const
+{
+    return get<QSize>("Interface/GridMajorSize", QSize(10, 10));
+}
+
 qreal Preferences::objectLineWidth() const
 {
     return get<qreal>("Interface/ObjectLineWidth", 2.0);
@@ -274,6 +319,12 @@ void Preferences::setShowObjectReferences(bool enabled)
     emit showObjectReferencesChanged(enabled);
 }
 
+void Preferences::setParallaxEnabled(bool enabled)
+{
+    setValue(QLatin1String("Interface/ParallaxEnabled"), enabled);
+    emit parallaxEnabledChanged(enabled);
+}
+
 void Preferences::setSnapToGrid(bool snapToGrid)
 {
     setValue(QLatin1String("Interface/SnapToGrid"), snapToGrid);
@@ -298,12 +349,34 @@ void Preferences::setGridColor(QColor gridColor)
     emit gridColorChanged(gridColor);
 }
 
+void Preferences::setBackgroundFadeColor(QColor backgroundFadeColor)
+{
+    setValue(QLatin1String("Interface/BackgroundFadeColor"), backgroundFadeColor.name());
+    emit backgroundFadeColorChanged(backgroundFadeColor);
+}
+
 void Preferences::setGridFine(int gridFine)
 {
     setValue(QLatin1String("Interface/GridFine"), gridFine);
     emit gridFineChanged(gridFine);
 }
 
+void Preferences::setGridMajor(QSize gridMajor)
+{
+    setValue(QLatin1String("Interface/GridMajorSize"), gridMajor);
+    emit gridMajorChanged(gridMajor);
+}
+
+void Preferences::setGridMajorX(int gridMajorX)
+{
+    setGridMajor(QSize(gridMajorX, gridMajor().height()));
+}
+
+void Preferences::setGridMajorY(int gridMajorY)
+{
+    setGridMajor(QSize(gridMajor().width(), gridMajorY));
+}
+
 void Preferences::setObjectLineWidth(qreal lineWidth)
 {
     setValue(QLatin1String("Interface/ObjectLineWidth"), lineWidth);
@@ -453,74 +526,10 @@ void Preferences::setObjectTypes(const ObjectTypes &objectTypes)
     emit objectTypesChanged();
 }
 
-static QString lastPathKey(Preferences::FileType fileType)
+void Preferences::setPropertyTypes(const SharedPropertyTypes &propertyTypes)
 {
-    QString key = QLatin1String("LastPaths/");
-
-    switch (fileType) {
-    case Preferences::ExportedFile:
-        key.append(QStringLiteral("ExportedFile"));
-        break;
-    case Preferences::ExternalTileset:
-        key.append(QStringLiteral("ExternalTileset"));
-        break;
-    case Preferences::ImageFile:
-        key.append(QStringLiteral("Images"));
-        break;
-    case Preferences::ObjectTemplateFile:
-        key.append(QStringLiteral("ObjectTemplates"));
-        break;
-    case Preferences::ObjectTypesFile:
-        key.append(QStringLiteral("ObjectTypes"));
-        break;
-    case Preferences::ProjectFile:
-        key.append(QStringLiteral("Project"));
-        break;
-    case Preferences::WorldFile:
-        key.append(QStringLiteral("WorldFile"));
-        break;
-    }
-
-    return key;
-}
-
-/**
- * Returns the last location of a file chooser for the given file type. As long
- * as it was set using setLastPath().
- *
- * When no last path for this file type exists yet, the path of the currently
- * selected map is returned.
- *
- * When no map is open, the user's 'Documents' folder is returned.
- */
-QString Preferences::lastPath(FileType fileType) const
-{
-    QString path = value(lastPathKey(fileType)).toString();
-
-    if (path.isEmpty()) {
-        DocumentManager *documentManager = DocumentManager::instance();
-        Document *document = documentManager->currentDocument();
-        if (document)
-            path = QFileInfo(document->fileName()).path();
-    }
-
-    if (path.isEmpty()) {
-        path = QStandardPaths::writableLocation(
-                    QStandardPaths::DocumentsLocation);
-    }
-
-    return path;
-}
-
-/**
- * \see lastPath()
- */
-void Preferences::setLastPath(FileType fileType, const QString &path)
-{
-    if (path.isEmpty())
-        return;
-
-    setValue(lastPathKey(fileType), path);
+    Object::setPropertyTypes(propertyTypes);
+    emit propertyTypesChanged();
 }
 
 QDate Preferences::firstRun() const
@@ -544,26 +553,26 @@ void Preferences::setPatron(bool isPatron)
     emit isPatronChanged();
 }
 
-bool Preferences::shouldShowDonationDialog() const
+bool Preferences::shouldShowDonationReminder() const
 {
     if (isPatron())
         return false;
     if (runCount() < 7)
         return false;
 
-    const QDate dialogTime = donationDialogTime();
+    const QDate dialogTime = donationReminderTime();
     if (!dialogTime.isValid())
         return false;
 
     return dialogTime.daysTo(QDate::currentDate()) >= 0;
 }
 
-QDate Preferences::donationDialogTime() const
+QDate Preferences::donationReminderTime() const
 {
     return get<QDate>("Install/DonationDialogTime");
 }
 
-void Preferences::setDonationDialogReminder(const QDate &date)
+void Preferences::setDonationReminder(const QDate &date)
 {
     if (date.isValid())
         setPatron(false);
@@ -584,10 +593,22 @@ QStringList Preferences::recentProjects() const
     return get<QStringList>("Project/RecentProjects");
 }
 
+QString Preferences::recentProjectPath() const
+{
+    QString path;
+
+    const auto recents = recentProjects();
+    if (!recents.isEmpty())
+        path = QFileInfo(recents.first()).path();
+
+    if (path.isEmpty())
+        path = homeLocation();
+
+    return path;
+}
+
 void Preferences::addRecentProject(const QString &fileName)
 {
-    setLastPath(ProjectFile, fileName);
-
     auto files = get<QStringList>("Project/RecentProjects");
     addToRecentFileList(fileName, files);
     setValue(QLatin1String("Project/RecentProjects"), files);
@@ -715,13 +736,25 @@ void Preferences::setWheelZoomsByDefault(bool mode)
     setValue(QLatin1String("Interface/WheelZoomsByDefault"), mode);
 }
 
-QString Preferences::dataLocation()
+QString Preferences::homeLocation()
 {
+    return QStandardPaths::writableLocation(QStandardPaths::HomeLocation);
+}
+
+QString Preferences::dataLocation() const
+{
+    if (mPortable) {
+        const auto configDir = QFileInfo(fileName()).dir();
+        return configDir.filePath(QStringLiteral("data"));
+    }
     return QStandardPaths::writableLocation(QStandardPaths::AppDataLocation);
 }
 
-QString Preferences::configLocation()
+QString Preferences::configLocation() const
 {
+    if (mPortable)
+        return QFileInfo(fileName()).path();
+
     return QStandardPaths::writableLocation(QStandardPaths::AppConfigLocation);
 }
 
@@ -781,3 +814,5 @@ void Preferences::objectTypesFileChangedOnDisk()
     if (ObjectTypesSerializer().readObjectTypes(fileInfo.filePath(), objectTypes))
         setObjectTypes(objectTypes);
 }
+
+#include "moc_preferences.cpp"
