@@ -22,6 +22,7 @@
 
 #include "brushitem.h"
 #include "editablemap.h"
+#include "editabletile.h"
 #include "mapdocument.h"
 #include "pluginmanager.h"
 #include "scriptmanager.h"
@@ -47,6 +48,14 @@ ScriptedTool::ScriptedTool(Id id, QJSValue object, QObject *parent)
     if (iconProperty.isString())
         setIconFileName(iconProperty.toString());
 
+    const QJSValue usesSelectedTilesProperty = mScriptObject.property(QStringLiteral("usesSelectedTiles"));
+    if (usesSelectedTilesProperty.isBool())
+        setUsesSelectedTiles(usesSelectedTilesProperty.toBool());
+
+    const QJSValue usesWangSetsProperty = mScriptObject.property(QStringLiteral("usesWangSets"));
+    if (usesWangSetsProperty.isBool())
+        setUsesWangSets(usesWangSetsProperty.toBool());
+
     // Make members of ScriptedTool available through the original object
     auto &scriptManager = ScriptManager::instance();
     auto self = scriptManager.engine()->newQObject(this);
@@ -69,7 +78,7 @@ EditableMap *ScriptedTool::editableMap() const
 EditableTile *ScriptedTool::editableTile() const
 {
     if (Tile *t = tile()) {
-        auto tileset = t->tileset()->sharedPointer();
+        auto tileset = t->tileset()->sharedFromThis();
 
         if (auto tilesetDocument = TilesetDocument::findDocumentForTileset(tileset)) {
             EditableTileset *editable = tilesetDocument->editable();
@@ -105,7 +114,6 @@ void ScriptedTool::setPreview(EditableMap *editableMap)
 void ScriptedTool::activate(MapScene *scene)
 {
     AbstractTileTool::activate(scene);
-    mScene = scene;
     call(QStringLiteral("activated"));
 }
 
@@ -113,7 +121,6 @@ void ScriptedTool::deactivate(MapScene *scene)
 {
     AbstractTileTool::deactivate(scene);
     call(QStringLiteral("deactivated"));
-    mScene = nullptr;
 }
 
 void ScriptedTool::keyPressed(QKeyEvent *keyEvent)
@@ -134,7 +141,7 @@ void ScriptedTool::mouseEntered()
 void ScriptedTool::mouseLeft()
 {
     AbstractTileTool::mouseLeft();
-    call(QStringLiteral("mouseEntered"));
+    call(QStringLiteral("mouseLeft"));
 }
 
 void ScriptedTool::mouseMoved(const QPointF &pos, Qt::KeyboardModifiers modifiers)
@@ -226,8 +233,11 @@ void ScriptedTool::setIconFileName(const QString &fileName)
 
     mIconFileName = fileName;
 
-    QString iconFile = QStringLiteral("ext:");
-    iconFile.append(fileName);
+    QString iconFile = fileName;
+
+    const QString ext = QStringLiteral("ext:");
+    if (!iconFile.startsWith(ext))
+        iconFile.prepend(ext);
 
     setIcon(QIcon { iconFile });
 }
@@ -285,3 +295,5 @@ bool ScriptedTool::call(const QString &methodName, const QJSValueList &args)
 }
 
 } // namespace Tiled
+
+#include "moc_scriptedtool.cpp"
