@@ -21,14 +21,16 @@
 #include "scriptmanager.h"
 
 #include "documentmanager.h"
+#include "editablegrouplayer.h"
+#include "editableimagelayer.h"
 #include "editablemap.h"
 #include "editablemapobject.h"
 #include "editableobjectgroup.h"
 #include "editableselectedarea.h"
-#include "editableterrain.h"
 #include "editabletile.h"
 #include "editabletilelayer.h"
 #include "editabletileset.h"
+#include "editablewangset.h"
 #include "logginginterface.h"
 #include "mapeditor.h"
 #include "mapview.h"
@@ -42,23 +44,30 @@
 #include "scriptfile.h"
 #include "scriptfileformatwrappers.h"
 #include "scriptfileinfo.h"
+#include "scriptimage.h"
 #include "scriptmodule.h"
+#include "scriptprocess.h"
 #include "tilecollisiondock.h"
 #include "tilelayer.h"
 #include "tilelayeredit.h"
 #include "tilesetdock.h"
 #include "tileseteditor.h"
 
+#include <QCoreApplication>
+#include <QDesktopServices>
 #include <QDir>
 #include <QFile>
 #include <QQmlEngine>
 #include <QStandardPaths>
+#if QT_VERSION < QT_VERSION_CHECK(6,0,0)
 #include <QTextCodec>
+#endif
 #include <QtDebug>
-#include <QCoreApplication>
 
 namespace Tiled {
 
+static Preference<QStringList> scriptingEnabledProjects { "Scripting/EnabledProjects" };
+
 ScriptManager *ScriptManager::mInstance;
 
 ScriptManager &ScriptManager::instance()
@@ -86,23 +95,26 @@ void ScriptManager::deleteInstance()
 ScriptManager::ScriptManager(QObject *parent)
     : QObject(parent)
 {
+    mResetTimer.setInterval(500);
+    mResetTimer.setSingleShot(true);
+    connect(&mResetTimer, &QTimer::timeout, this, &ScriptManager::reset);
+
     qRegisterMetaType<Cell>();
     qRegisterMetaType<EditableAsset*>();
+    qRegisterMetaType<EditableGroupLayer*>();
     qRegisterMetaType<EditableLayer*>();
     qRegisterMetaType<EditableMap*>();
     qRegisterMetaType<EditableMapObject*>();
     qRegisterMetaType<EditableObjectGroup*>();
     qRegisterMetaType<EditableSelectedArea*>();
-    qRegisterMetaType<EditableTerrain*>();
     qRegisterMetaType<EditableTile*>();
     qRegisterMetaType<EditableTileLayer*>();
     qRegisterMetaType<EditableTileset*>();
+    qRegisterMetaType<EditableWangSet*>();
     qRegisterMetaType<Font>();
     qRegisterMetaType<MapEditor*>();
     qRegisterMetaType<MapView*>();
     qRegisterMetaType<RegionValueType>();
-    qRegisterMetaType<ScriptBinaryFile*>();
-    qRegisterMetaType<ScriptTextFile*>();
     qRegisterMetaType<ScriptedAction*>();
     qRegisterMetaType<ScriptedTool*>();
     qRegisterMetaType<TileCollisionDock*>();
@@ -111,6 +123,7 @@ ScriptManager::ScriptManager(QObject *parent)
     qRegisterMetaType<TilesetEditor*>();
     qRegisterMetaType<ScriptMapFormatWrapper*>();
     qRegisterMetaType<ScriptTilesetFormatWrapper*>();
+    qRegisterMetaType<ScriptImage*>();
 
     connect(&mWatcher, &FileSystemWatcher::pathsChanged,
             this, &ScriptManager::scriptFilesChanged);
@@ -118,7 +131,7 @@ ScriptManager::ScriptManager(QObject *parent)
     connect(ProjectManager::instance(), &ProjectManager::projectChanged,
             this, &ScriptManager::refreshExtensionsPaths);
 
-    const QString configLocation { Preferences::configLocation() };
+    const QString configLocation { Preferences::instance()->configLocation() };
     if (!configLocation.isEmpty()) {
         mExtensionsPath = QDir{configLocation}.filePath(QStringLiteral("extensions"));
 
@@ -151,6 +164,7 @@ QJSValue ScriptManager::evaluate(const QString &program,
     return result;
 }
 
+#if QT_VERSION < QT_VERSION_CHECK(6,0,0)
 static bool fromUtf8(const QByteArray &bytes, QString &unicode)
 {
     QTextCodec::ConverterState state;
@@ -158,6 +172,7 @@ static bool fromUtf8(const QByteArray &bytes, QString &unicode)
     unicode = codec->toUnicode(bytes.constData(), bytes.size(), &state);
     return state.invalidChars == 0;
 }
+#endif
 
 QJSValue ScriptManager::evaluateFile(const QString &fileName)
 {
@@ -170,8 +185,18 @@ QJSValue ScriptManager::evaluateFile(const QString &fileName)
 
     const QByteArray bytes = file.readAll();
     QString script;
+#if QT_VERSION < QT_VERSION_CHECK(6,0,0)
     if (!fromUtf8(bytes, script))
         script = QTextCodec::codecForUtfText(bytes)->toUnicode(bytes);
+#else
+    auto encoding = QStringConverter::encodingForData(bytes.constData(), bytes.size());
+    QStringDecoder decoder(encoding.value_or(QStringConverter::Utf8));
+    script = decoder.decode(bytes);
+    if (decoder.hasError()) {
+        Tiled::ERROR(tr("Error decoding file: %1").arg(fileName));
+        return QJSValue();
+    }
+#endif
 
     Tiled::INFO(tr("Evaluating '%1'").arg(fileName));
     return evaluate(script, fileName);
@@ -209,13 +234,53 @@ void ScriptManager::loadExtension(const QString &path)
 {
     mWatcher.addPath(path);
 
+    const QStringList nameFilters = {
+        QLatin1String("*.js"),
+        QLatin1String("*.mjs")
+    };
     const QDir dir(path);
-    const QStringList jsFiles = dir.entryList({ QLatin1String("*.js") },
+    const QStringList jsFiles = dir.entryList(nameFilters,
                                               QDir::Files | QDir::Readable);
 
+#if QT_VERSION < QT_VERSION_CHECK(5, 12, 0)
+    bool hasWarned = false;
+#endif
+
     for (const QString &jsFile : jsFiles) {
         const QString absolutePath = dir.filePath(jsFile);
-        evaluateFile(absolutePath);
+        if (absolutePath.endsWith(QLatin1String(".js"), Qt::CaseInsensitive)) {
+            evaluateFile(absolutePath);
+        } else {
+#if QT_VERSION < QT_VERSION_CHECK(5, 12, 0)
+            if (!hasWarned) {
+                Tiled::WARNING(tr("Importing modules (%1) not supported by this version of Tiled").arg(absolutePath));
+                hasWarned = true;
+            }
+#else
+            Tiled::INFO(tr("Importing module '%1'").arg(absolutePath));
+
+            QJSValue globalObject = mEngine->globalObject();
+            globalObject.setProperty(QStringLiteral("__filename"), absolutePath);
+
+            QJSValue result = mEngine->importModule(absolutePath);
+
+            // According to the documentation, importModule could return an
+            // error object, though in practice this doesn't appear to happen.
+            if (!checkError(result)) {
+#if QT_VERSION >= QT_VERSION_CHECK(6, 1, 0)
+                // This appears to be the way to report exceptions
+                checkError(mEngine->catchError());
+#else
+                // With Qt 5 it seems we need to get a little creative, like
+                // calling evaluate to let that catch a potentially raised
+                // exception.
+                checkError(mEngine->evaluate(QString()));
+#endif
+            }
+
+            globalObject.deleteProperty(QStringLiteral("__filename"));
+#endif
+        }
         mWatcher.addPath(absolutePath);
     }
 }
@@ -228,7 +293,11 @@ bool ScriptManager::checkError(QJSValue value, const QString &program)
     QString errorString = value.toString();
     QString stack = value.property(QStringLiteral("stack")).toString();
 
+#if QT_VERSION >= QT_VERSION_CHECK(6,0,0)
+    const auto stackEntries = QStringView(stack).split(QLatin1Char('\n'));
+#else
     const auto stackEntries = stack.splitRef(QLatin1Char('\n'));
+#endif
     if (stackEntries.size() > 0 && !stackEntries.first().startsWith(QLatin1String("%entry@"))) {
         // Add stack if there were more than one entries
         errorString.append(QLatin1Char('\n'));
@@ -270,6 +339,13 @@ void ScriptManager::throwNullArgError(int argNumber)
 
 void ScriptManager::reset()
 {
+    // If resetting the script engine is currently blocked, which can happen
+    // while a script is waiting for a popup, try again later.
+    if (mResetBlocked) {
+        mResetTimer.start();
+        return;
+    }
+
     Tiled::INFO(tr("Resetting script engine"));
 
     mWatcher.clear();
@@ -286,29 +362,48 @@ void ScriptManager::reset()
 
 void ScriptManager::initialize()
 {
-    mEngine = new QQmlEngine(this);
+    auto engine = new QQmlEngine(this);
+
+    // We'll report errors in the Console view instead
+    engine->setOutputWarningsToStandardError(false);
+    connect(engine, &QQmlEngine::warnings, this, &ScriptManager::onScriptWarnings);
+
+    mEngine = engine;
     mModule = new ScriptModule(this);
 
-    QJSValue globalObject = mEngine->globalObject();
-    globalObject.setProperty(QStringLiteral("tiled"), mEngine->newQObject(mModule));
+    QJSValue globalObject = engine->globalObject();
+    globalObject.setProperty(QStringLiteral("tiled"), engine->newQObject(mModule));
 #if QT_VERSION >= 0x050800
-    globalObject.setProperty(QStringLiteral("TextFile"), mEngine->newQMetaObject<ScriptTextFile>());
-    globalObject.setProperty(QStringLiteral("BinaryFile"), mEngine->newQMetaObject<ScriptBinaryFile>());
-    globalObject.setProperty(QStringLiteral("Layer"), mEngine->newQMetaObject<EditableLayer>());
-    globalObject.setProperty(QStringLiteral("MapObject"), mEngine->newQMetaObject<EditableMapObject>());
-    globalObject.setProperty(QStringLiteral("ObjectGroup"), mEngine->newQMetaObject<EditableObjectGroup>());
-    globalObject.setProperty(QStringLiteral("Terrain"), mEngine->newQMetaObject<EditableTerrain>());
-    globalObject.setProperty(QStringLiteral("Tile"), mEngine->newQMetaObject<EditableTile>());
-    globalObject.setProperty(QStringLiteral("TileLayer"), mEngine->newQMetaObject<EditableTileLayer>());
-    globalObject.setProperty(QStringLiteral("TileMap"), mEngine->newQMetaObject<EditableMap>());
-    globalObject.setProperty(QStringLiteral("Tileset"), mEngine->newQMetaObject<EditableTileset>());
+    globalObject.setProperty(QStringLiteral("GroupLayer"), engine->newQMetaObject<EditableGroupLayer>());
+    globalObject.setProperty(QStringLiteral("Image"), engine->newQMetaObject<ScriptImage>());
+    globalObject.setProperty(QStringLiteral("ImageLayer"), engine->newQMetaObject<EditableImageLayer>());
+    globalObject.setProperty(QStringLiteral("Layer"), engine->newQMetaObject<EditableLayer>());
+    globalObject.setProperty(QStringLiteral("MapObject"), engine->newQMetaObject<EditableMapObject>());
+    globalObject.setProperty(QStringLiteral("ObjectGroup"), engine->newQMetaObject<EditableObjectGroup>());
+    globalObject.setProperty(QStringLiteral("Tile"), engine->newQMetaObject<EditableTile>());
+    globalObject.setProperty(QStringLiteral("TileLayer"), engine->newQMetaObject<EditableTileLayer>());
+    globalObject.setProperty(QStringLiteral("TileMap"), engine->newQMetaObject<EditableMap>());
+    globalObject.setProperty(QStringLiteral("Tileset"), engine->newQMetaObject<EditableTileset>());
+    globalObject.setProperty(QStringLiteral("WangSet"), engine->newQMetaObject<EditableWangSet>());
 #endif
 
-    registerFileInfo(mEngine);
+    registerFile(engine);
+    registerFileInfo(engine);
+    registerProcess(engine);
 
     loadExtensions();
 }
 
+void ScriptManager::onScriptWarnings(const QList<QQmlError> &warnings)
+{
+    for (const auto &warning : warnings) {
+        Tiled::ERROR(warning.toString(), [url = warning.url()] {
+            if (!url.isEmpty())
+                QDesktopServices::openUrl(url);
+        });
+    }
+}
+
 void ScriptManager::scriptFilesChanged(const QStringList &scriptFiles)
 {
     Tiled::INFO(tr("Script files changed: %1").arg(scriptFiles.join(QLatin1String(", "))));
@@ -323,11 +418,21 @@ void ScriptManager::refreshExtensionsPaths()
         extensionsPaths.append(mExtensionsPath);
 
     // Add extensions path from project
-    auto &projectExtensionsPath = ProjectManager::instance()->project().mExtensionsPath;
-    if (!projectExtensionsPath.isEmpty()) {
-        const QFileInfo info(projectExtensionsPath);
-        if (info.exists() && info.isDir())
-            extensionsPaths.append(projectExtensionsPath);
+    bool projectExtensionsSuppressed = false;
+    const Project &project = ProjectManager::instance()->project();
+    if (!project.mExtensionsPath.isEmpty()) {
+        const QFileInfo info(project.mExtensionsPath);
+        if (info.exists() && info.isDir()) {
+            if (scriptingEnabledProjects.get().contains(project.fileName(), Qt::CaseInsensitive))
+                extensionsPaths.append(project.mExtensionsPath);
+            else
+                projectExtensionsSuppressed = true;
+        }
+    }
+
+    if (mProjectExtensionsSuppressed != projectExtensionsSuppressed) {
+        mProjectExtensionsSuppressed = projectExtensionsSuppressed;
+        emit projectExtensionsSuppressedChanged(projectExtensionsSuppressed);
     }
 
     extensionsPaths.sort();
@@ -344,4 +449,21 @@ void ScriptManager::refreshExtensionsPaths()
     }
 }
 
+void ScriptManager::enableProjectExtensions()
+{
+    const Project &project = ProjectManager::instance()->project();
+    const QString &fileName = project.fileName();
+    if (!fileName.isEmpty()) {
+        QStringList projects = scriptingEnabledProjects;
+        if (!projects.contains(fileName, Qt::CaseInsensitive)) {
+            projects.append(fileName);
+            scriptingEnabledProjects = projects;
+
+            refreshExtensionsPaths();
+        }
+    }
+}
+
 } // namespace Tiled
+
+#include "moc_scriptmanager.cpp"
