@@ -64,6 +64,8 @@ ScriptModule::~ScriptModule()
     for (const auto &pair : mRegisteredActions)
         ActionManager::unregisterAction(pair.second.get(), pair.first);
 
+    ActionManager::clearMenuExtensions();
+
     IssuesModel::instance().removeIssuesWithContext(this);
 }
 
@@ -96,6 +98,16 @@ QString ScriptModule::arch() const
 #endif
 }
 
+QString ScriptModule::extensionsPath() const
+{
+    return ScriptManager::instance().extensionsPath();
+}
+
+QString ScriptModule::applicationDirPath() const
+{
+    return QCoreApplication::applicationDirPath();
+}
+
 static QStringList idsToNames(const QList<Id> &ids)
 {
     QStringList names;
@@ -161,9 +173,16 @@ bool ScriptModule::setActiveAsset(EditableAsset *asset) const
         return false;
     }
 
-    for (const DocumentPtr &document : documentManager->documents())
-        if (document->editable() == asset)
-            return documentManager->switchToDocument(document.data());
+    if (asset->checkReadOnly())
+        return false;
+
+    if (auto document = asset->document())
+        return documentManager->switchToDocument(document);
+
+    if (auto document = asset->createDocument()) {
+        documentManager->addDocument(document);
+        return true;
+    }
 
     return false;
 }
@@ -201,6 +220,26 @@ ObjectRef ScriptModule::objectRef(int id) const
     return { id };
 }
 
+QVariant ScriptModule::propertyValue(const QString &typeName, const QVariant &value) const
+{
+    auto type = Object::propertyTypes().findTypeByName(typeName);
+    if (!type) {
+        ScriptManager::instance().throwError(QCoreApplication::translate("Script Errors", "Unknown type: %1").arg(typeName));
+        return {};
+    }
+
+    switch (type->type) {
+    case PropertyType::PT_Invalid:
+    case PropertyType::PT_Class:
+        break;
+    case PropertyType::PT_Enum:
+        // Call toPropertyValue to support using strings to create a value
+        return type->toPropertyValue(value, ExportContext());
+    }
+
+    return type->wrap(value);
+}
+
 EditableAsset *ScriptModule::open(const QString &fileName) const
 {
     auto documentManager = DocumentManager::maybeInstance();
@@ -412,16 +451,16 @@ static Id toId(QJSValue value)
 
 void ScriptModule::extendMenu(const QByteArray &idName, QJSValue items)
 {
-    MenuExtension extension;
-    extension.menuId = Id(idName);
+    ActionManager::MenuExtension extension;
+    Id menuId(idName);
 
-    if (!ActionManager::findMenu(extension.menuId)) {
+    if (!ActionManager::hasMenu(menuId)) {
         ScriptManager::instance().throwError(QCoreApplication::translate("Script Errors", "Unknown menu"));
         return;
     }
 
     auto addItem = [&] (QJSValue item) -> bool {
-        MenuItem menuItem;
+        ActionManager::MenuItem menuItem;
 
         const QJSValue action = item.property(QStringLiteral("action"));
 
@@ -459,21 +498,7 @@ void ScriptModule::extendMenu(const QByteArray &idName, QJSValue items)
         return;
     }
 
-    // Apply the extension
-    QMenu *menu = ActionManager::menu(extension.menuId);
-    QAction *before = nullptr;
-
-    for (MenuItem &item : extension.items) {
-        if (item.beforeAction)
-            before = ActionManager::findAction(item.beforeAction);
-
-        if (item.isSeparator)
-            menu->insertSeparator(before)->setParent(this);
-        else
-            menu->insertAction(before, ActionManager::action(item.action));
-    }
-
-    mMenuExtensions.append(extension);
+    ActionManager::registerMenuExtension(menuId, extension);
 }
 
 void ScriptModule::trigger(const QByteArray &actionName) const
@@ -500,16 +525,36 @@ void ScriptModule::executeCommand(const QString &name, bool inTerminal) const
 
 void ScriptModule::alert(const QString &text, const QString &title) const
 {
-    QMessageBox::warning(MainWindow::instance(), title, text);
+    ScriptManager::ResetBlocker blocker;
+    QMessageBox msgBox(QMessageBox::Warning, title, text, QMessageBox::Ok,
+                       MainWindow::instance());
+
+    // On macOS, QMessageBox hides the title by default
+#ifdef Q_OS_MAC
+    msgBox.QDialog::setWindowTitle(title);
+#endif
+
+    msgBox.exec();
 }
 
 bool ScriptModule::confirm(const QString &text, const QString &title) const
 {
-    return QMessageBox::question(MainWindow::instance(), title, text) == QMessageBox::Yes;
+    ScriptManager::ResetBlocker blocker;
+    QMessageBox msgBox(QMessageBox::Question, title, text,
+                       QMessageBox::Yes | QMessageBox::No,
+                       MainWindow::instance());
+
+    // On macOS, QMessageBox hides the title by default
+#ifdef Q_OS_MAC
+    msgBox.QDialog::setWindowTitle(title);
+#endif
+
+    return msgBox.exec() == QMessageBox::Yes;
 }
 
 QString ScriptModule::prompt(const QString &label, const QString &text, const QString &title) const
 {
+    ScriptManager::ResetBlocker blocker;
     return QInputDialog::getText(MainWindow::instance(), title, label, QLineEdit::Normal, text);
 }
 
@@ -574,3 +619,5 @@ void ScriptModule::currentDocumentChanged(Document *document)
 }
 
 } // namespace Tiled
+
+#include "moc_scriptmodule.cpp"
