@@ -20,20 +20,21 @@
 
 #include "tilesetdocument.h"
 
+#include "changeevents.h"
+#include "editablemanager.h"
 #include "editabletileset.h"
 #include "issuesmodel.h"
 #include "map.h"
 #include "mapdocument.h"
-#include "terrain.h"
 #include "tile.h"
 #include "tilesetformat.h"
-#include "tilesetterrainmodel.h"
 #include "tilesetwangsetmodel.h"
 #include "wangcolormodel.h"
 #include "wangset.h"
 
 #include <QCoreApplication>
 #include <QFileInfo>
+#include <QQmlEngine>
 #include <QUndoStack>
 
 namespace Tiled {
@@ -62,12 +63,18 @@ QMap<SharedTileset, TilesetDocument*> TilesetDocument::sTilesetToDocument;
 TilesetDocument::TilesetDocument(const SharedTileset &tileset)
     : Document(TilesetDocumentType, tileset->fileName())
     , mTileset(tileset)
-    , mTerrainModel(new TilesetTerrainModel(this, this))
     , mWangSetModel(new TilesetWangSetModel(this, this))
 {
     Q_ASSERT(!sTilesetToDocument.contains(tileset));
     sTilesetToDocument.insert(tileset, this);
 
+    // If there already happens to be an editable for this tileset, take
+    // ownership of it.
+    if (auto editable = EditableManager::instance().find(tileset.data())) {
+        setEditable(std::unique_ptr<EditableAsset>(editable));
+        QQmlEngine::setObjectOwnership(editable, QQmlEngine::CppOwnership);
+    }
+
     mCurrentObject = tileset.data();
 
     connect(this, &TilesetDocument::propertyAdded,
@@ -79,9 +86,6 @@ TilesetDocument::TilesetDocument(const SharedTileset &tileset)
     connect(this, &TilesetDocument::propertiesChanged,
             this, &TilesetDocument::onPropertiesChanged);
 
-    connect(mTerrainModel, &TilesetTerrainModel::terrainRemoved,
-            this, &TilesetDocument::onTerrainRemoved);
-
     connect(mWangSetModel, &TilesetWangSetModel::wangSetRemoved,
             this, &TilesetDocument::onWangSetRemoved);
 }
@@ -101,10 +105,12 @@ TilesetDocument::~TilesetDocument()
 
 bool TilesetDocument::save(const QString &fileName, QString *error)
 {
-    TilesetFormat *tilesetFormat = mTileset->format();
-
-    if (!tilesetFormat || !(tilesetFormat->capabilities() & FileFormat::Write))
+    auto tilesetFormat = findFileFormat<TilesetFormat>(mTileset->format(), FileFormat::Write);;
+    if (!tilesetFormat) {
+        if (error)
+            *error = tr("Tileset format '%s' not found").arg(mTileset->format());
         return false;
+    }
 
     if (!tilesetFormat->write(*tileset(), fileName)) {
         if (error)
@@ -129,7 +135,7 @@ bool TilesetDocument::save(const QString &fileName, QString *error)
 
 bool TilesetDocument::canReload() const
 {
-    return !fileName().isEmpty() && mTileset->format();
+    return !fileName().isEmpty() && !mTileset->format().isEmpty();
 }
 
 bool TilesetDocument::reload(QString *error)
@@ -137,7 +143,12 @@ bool TilesetDocument::reload(QString *error)
     if (!canReload())
         return false;
 
-    auto format = mTileset->format();
+    auto format = findFileFormat<TilesetFormat>(mTileset->format(), FileFormat::Read);
+    if (!format) {
+        if (error)
+            *error = tr("Tileset format '%s' not found").arg(mTileset->format());
+        return false;
+    }
 
     SharedTileset tileset = format->read(fileName());
 
@@ -148,7 +159,7 @@ bool TilesetDocument::reload(QString *error)
     }
 
     tileset->setFileName(fileName());
-    tileset->setFormat(format);
+    tileset->setFormat(format->shortName());
 
     undoStack()->push(new ReloadTileset(this, tileset));
     undoStack()->setClean();
@@ -170,19 +181,20 @@ TilesetDocumentPtr TilesetDocument::load(const QString &fileName,
     }
 
     tileset->setFileName(fileName);
-    tileset->setFormat(format);
+    tileset->setFormat(format->shortName());
 
     return TilesetDocumentPtr::create(tileset);
 }
 
-FileFormat *TilesetDocument::writerFormat() const
+TilesetFormat *TilesetDocument::writerFormat() const
 {
-    return mTileset->format();
+    return findFileFormat<TilesetFormat>(mTileset->format(), FileFormat::Write);
 }
 
 void TilesetDocument::setWriterFormat(TilesetFormat *format)
 {
-    mTileset->setFormat(format);
+    Q_ASSERT(format->hasCapabilities(FileFormat::Write));
+    mTileset->setFormat(format->shortName());
 }
 
 QString TilesetDocument::lastExportFileName() const
@@ -197,8 +209,6 @@ void TilesetDocument::setLastExportFileName(const QString &fileName)
 
 TilesetFormat* TilesetDocument::exportFormat() const
 {
-    if (tileset()->exportFormat.isEmpty())
-        return nullptr;
     return findFileFormat<TilesetFormat>(tileset()->exportFormat);
 }
 
@@ -258,12 +268,9 @@ void TilesetDocument::swapTileset(SharedTileset &tileset)
     emit tilesetChanged(mTileset.data());
 }
 
-EditableTileset *TilesetDocument::editable()
+std::unique_ptr<EditableAsset> TilesetDocument::createEditable()
 {
-    if (!mEditable)
-        mEditable.reset(new EditableTileset(this, this));
-
-    return static_cast<EditableTileset*>(mEditable.get());
+    return std::make_unique<EditableTileset>(this, this);
 }
 
 /**
@@ -319,6 +326,12 @@ void TilesetDocument::setTilesetObjectAlignment(Alignment objectAlignment)
         emit mapDocument->tilesetTilePositioningChanged(mTileset.data());
 }
 
+void TilesetDocument::setTilesetTransformationFlags(Tileset::TransformationFlags flags)
+{
+    tileset()->setTransformationFlags(flags);
+    emit tilesetChanged(mTileset.data());
+}
+
 void TilesetDocument::addTiles(const QList<Tile *> &tiles)
 {
     mTileset->addTiles(tiles);
@@ -336,11 +349,22 @@ void TilesetDocument::removeTiles(const QList<Tile *> &tiles)
         }
     }
 
+    emit changed(TilesEvent(ChangeEvent::TilesAboutToBeRemoved, tiles));
     mTileset->removeTiles(tiles);
     emit tilesRemoved(tiles);
     emit tilesetChanged(mTileset.data());
 }
 
+/**
+ * \sa Tileset::relocateTiles
+ */
+QList<int> TilesetDocument::relocateTiles(const QList<Tile *> &tiles, int location)
+{
+    const auto prevLocations = mTileset->relocateTiles(tiles, location);
+    emit tilesetChanged(mTileset.data());
+    return prevLocations;
+}
+
 void TilesetDocument::setSelectedTiles(const QList<Tile*> &selectedTiles)
 {
     mSelectedTiles = selectedTiles;
@@ -438,8 +462,6 @@ void TilesetDocument::checkIssues()
                   std::function<void()>(), this);   // todo: hook to file dialog
         }
     }
-    for (Terrain *terrain : tileset()->terrains())
-        checkFilePathProperties(terrain);
     for (WangSet *wangSet : tileset()->wangSets()) {
         checkFilePathProperties(wangSet);
         // todo: check properties on wang colors
@@ -475,18 +497,11 @@ void TilesetDocument::onPropertiesChanged(Object *object)
         emit mapDocument->propertiesChanged(object);
 }
 
-void TilesetDocument::onTerrainRemoved(Terrain *terrain)
-{
-    if (terrain == mCurrentObject)
-        setCurrentObject(nullptr);
-}
-
 void TilesetDocument::onWangSetRemoved(WangSet *wangSet)
 {
-    if (wangSet == mCurrentObject)
-        setCurrentObject(nullptr);
-
     mWangColorModels.erase(wangSet);
 }
 
 } // namespace Tiled
+
+#include "moc_tilesetdocument.cpp"
