@@ -1,7 +1,7 @@
 /*
  * varianteditorfactory.cpp
  * Copyright (C) 2006 Trolltech ASA. All rights reserved. (GPLv2)
- * Copyright 2013, Thorbjørn Lindeijer <thorbjorn@lindeijer.nl>
+ * Copyright 2013-2021, Thorbjørn Lindeijer <thorbjorn@lindeijer.nl>
  *
  * This file is part of Tiled.
  *
@@ -66,7 +66,7 @@ ResetWidget::ResetWidget(QtProperty *property, QWidget *editor, QWidget *parent)
     resetButton->setToolTip(tr("Reset"));
     Utils::setThemeIcon(resetButton, "edit-clear");
 
-    layout->setMargin(0);
+    layout->setContentsMargins(0, 0, 0, 0);
     layout->setSpacing(0);
     layout->addWidget(editor);
     layout->addWidget(resetButton);
@@ -155,7 +155,7 @@ QWidget *VariantEditorFactory::createEditor(QtVariantPropertyManager *manager,
         return editor;
     }
 
-    if (type == QVariant::String) {
+    if (type == QMetaType::QString) {
         bool multiline = manager->attributeValue(property, QLatin1String("multiline")).toBool();
         if (multiline) {
             auto editor = new TextPropertyEdit(parent);
@@ -191,8 +191,9 @@ QWidget *VariantEditorFactory::createEditor(QtVariantPropertyManager *manager,
 
     QWidget *editor = QtVariantEditorFactory::createEditor(manager, property, parent);
 
-    if (type == QVariant::Color) {
-        // Allow resetting a color property to the invalid color
+    if (type == QMetaType::QColor || property->isModified()) {
+        // Allow resetting a color property to the invalid color, or allow
+        // unsetting a class member (todo: resolve conflict...).
         ResetWidget *resetWidget = new ResetWidget(property, editor, parent);
         connect(resetWidget, &ResetWidget::resetProperty,
                 this, &VariantEditorFactory::resetProperty);
@@ -384,3 +385,4 @@ void VariantEditorFactory::slotEditorDestroyed(QObject *object)
 } // namespace Tiled
 
 #include "varianteditorfactory.moc"
+#include "moc_varianteditorfactory.cpp"
