@@ -368,6 +368,20 @@ void WangDock::setDocument(Document *document)
     }
 }
 
+int WangDock::currentWangColor() const
+{
+    QItemSelectionModel *selectionModel = mWangColorView->selectionModel();
+    const auto currentIndex = selectionModel->currentIndex();
+    int color = 0;
+
+    if (currentIndex.isValid()) {
+        QModelIndex index = static_cast<QAbstractProxyModel*>(mWangColorView->model())->mapToSource(currentIndex);
+        color = mWangColorModel->colorAt(index);
+    }
+
+    return color;
+}
+
 void WangDock::editWangSetName(WangSet *wangSet)
 {
     const QModelIndex index = wangSetIndex(wangSet);
@@ -434,14 +448,7 @@ void WangDock::refreshCurrentWangId()
 
 void WangDock::refreshCurrentWangColor()
 {
-    QItemSelectionModel *selectionModel = mWangColorView->selectionModel();
-    const auto currentIndex = selectionModel->currentIndex();
-    int color = 0;
-
-    if (currentIndex.isValid()) {
-        QModelIndex index = static_cast<QAbstractProxyModel*>(mWangColorView->model())->mapToSource(currentIndex);
-        color = mWangColorModel->colorAt(index);
-    }
+    const int color = currentWangColor();
 
     mEraseWangIdsButton->setChecked(color == 0);
     mRemoveColor->setEnabled(color != 0);
@@ -472,10 +479,15 @@ void WangDock::documentChanged(const ChangeEvent &change)
     }
 }
 
-void WangDock::wangSetChanged()
+void WangDock::wangSetChanged(WangSet *wangSet)
 {
-    mWangColorModel->resetModel();
-    mWangColorView->expandAll();
+    if (mCurrentWangSet != wangSet)
+        return;
+
+    if (mWangColorModel) {
+        mWangColorModel->resetModel();
+        mWangColorView->expandAll();
+    }
 
     refreshCurrentWangColor();
     refreshCurrentWangId();
@@ -546,7 +558,7 @@ void WangDock::setCurrentWangSet(WangSet *wangSet)
     TilesetDocument *tilesetDocument = nullptr;
 
     if (wangSet) {
-        auto sharedTileset = wangSet->tileset()->sharedPointer();
+        auto sharedTileset = wangSet->tileset()->sharedFromThis();
         auto documentManager = DocumentManager::instance();
 
         tilesetDocument = documentManager->findTilesetDocument(sharedTileset);
